#!/bin/sh
#
# Gather all of the firmware files referenced by the available kernel
# modules.
#

SYSROOT="$1"
KVER="$2"
DST="$3"
WRKSRC="$4"

PATH=${SYSROOT}/usr/bin:${SYSROOT}/usr/sbin

# the kernel module directory will have the package build number, which we have to find
KDIR="$(basename "$(ls -1d "${SYSROOT}"/usr/lib/modules/"${KVER}"-*)" 2>/dev/null)"

if [ -z "${KDIR}" ]; then
    echo "*** Unable to find kernel modules" >&2
    exit 1
fi

FWLIST="$(mktemp)"
trap 'rm -f "${FWLIST}"' EXIT

# gather all of the firmware filenames
find "${SYSROOT}"/usr/lib/modules/"${KDIR}" -type f -name "*.ko*" | while read -r modname ; do
    modinfo "${modname}" 2>/dev/null | grep ^firmware: | awk '{ print $2; }' | while read -r fwfile ; do
        chkfile="$(realpath -e "${WRKSRC}"/"${fwfile}" 2>/dev/null)"
        if [ -f "${chkfile}" ]; then
            echo "${fwfile}" >> "${FWLIST}"
        else
            echo "excluding ${fwfile}"
        fi
    done
done

sort "${FWLIST}" | uniq > "${FWLIST}".new
mv "${FWLIST}".new "${FWLIST}"

# create the destination directory
[ -d "${DST}" ] || mkdir -p "${DST}"

# use tar to copy since some filenames are in subdirectories
cd "${WRKSRC}" || exit 1
tar -cvf - -T "${FWLIST}" | ( cd "${DST}" || exit 1 ; tar -xf - )
