#!/bin/sh
# Copyright David Cantrell <dcantrell@burdell.org>
# SPDX-License-Identifier: Apache-2.0

# Generate version number variables for use in the package Makefile
# For example, a block like this:
#
#     [Archive-Zip]
#     ver: 1.31_04
#     urls: http://www.cpan.org/authors/id/A/AD/ADAMK/
#     regexp: Archive-Zip-[0-9\._]+\.tar
#     prefix: Archive-Zip-
#
# Will have a variable definition like this:
#
#     ARCHIVE_ZIP_VER = 1.31_04
#
# Like the Haskell packages, this means we just need to maintain the
# version numbers in the upstream file.

PATH=/bin:/usr/bin
CWD="$(pwd)"
UPSTREAM="${CWD}/upstream"
WRKDIR="${1}"
PERLVERS="${WRKDIR}/perlvers.mk"

if [ ! -f "${UPSTREAM}" ]; then
    echo "*** missing ${UPSTREAM}, exiting" >&2
    exit 1
fi

if [ -z "${WRKDIR}" ]; then
    echo "*** missing ${WRKDIR}, exiting" >&2
    exit 1
fi

[ -d "${WRKDIR}" ] || mkdir -p "${WRKDIR}"
[ -f "${PERLVERS}" ] && rm -f "${PERLVERS}"

component=
version=

while read -r line ; do
    [ -z "${line}" ] && continue

    # get first and last line characters
    first="$(echo "${line}" | cut -c 1)"
    last="$(echo "${line}" | rev | cut -c 1)"
    field="$(echo "${line}" | cut -d ':' -f 1)"

    if [ "${first}" = "[" ] && [ "${last}" = "]" ]; then
        component="$(echo "${line}" | sed -e 's|\[||g' -e 's|\]||g' -e 's|[\.\-]|_|g' | tr '[:lower:]' '[:upper:]')_VER"
    elif [ -n "${component}" ] && [ "${field}" = "ver" ]; then
        version="$(echo "${line}" | cut -d ':' -f 2 | sed -r 's|\s*||g')"
        echo "${component} = ${version}" >> "${PERLVERS}"
        component=
        version=
    fi
done < "${UPSTREAM}"

echo "${PERLVERS}"
