#!/usr/bin/python3
#
# Copyright Chris Lumens <chris@bangmoney.org>
# Copyright David Cantrell <dcantrell@burdell.org>
# SPDX-License-Identifier: Apache-2.0
#

import os
import re
import shutil
import sys

if sys.version_info.major == 2:
    import ConfigParser as configparser
else:
    import configparser

UPSTREAM_FILE = 'upstream'

if __name__ == "__main__":
    upstream = configparser.ConfigParser()
    upstream.read(UPSTREAM_FILE)

    if len(sys.argv) != 2:
        sys.stderr.write("Usage: %s [workdir]\n" % os.path.basename(sys.argv[0]))
        sys.exit(1)

    wrkdir = sys.argv[1]

    # iterate over each upstream source
    for section in upstream.sections():
        # skip the main Python section
        if section == 'python':
            continue

        # we may be the upstream, so just continue
        if upstream.has_option(section, 'no_upstream') and upstream.getboolean(section, 'no_upstream'):
            continue

        if upstream.has_option(section, 'ver'):
            moduledir = section
            target = os.path.join(wrkdir, moduledir + '-' + upstream.get(section, 'ver'))
            dest = os.path.join(wrkdir, section)
        else:
            continue

        for subdir in os.listdir(wrkdir):
            fpath = os.path.join(wrkdir, subdir)

            if not os.path.isdir(fpath):
                continue

            if subdir == moduledir or subdir.find(moduledir) != -1:
                target = os.path.join(wrkdir, subdir)
                break

        if os.path.islink(dest):
            os.unlink(dest)
        elif os.path.isdir(dest):
            shutil.rmtree(dest, ignore_errors=True)

        os.symlink(target, dest)
