#!/bin/sh
#
# Rewrite the DT_INTERP value on the prebuilt bootstrap files for
# rust.
#
# Usage:
# fix-DT_INTERP.sh /path/to/name-of-file.tar.gz gz
#
# That is:  [script] [tar file full path] [compression extension]
#

PATH=/usr/bin
DESTFILE="$1"
ZEXT="$2"
DNAME="$(basename "${DESTFILE}" .tar."${ZEXT}")"

TMPDIR="$(mktemp -d)"
trap 'rm -rf "${TMPDIR}"' EXIT

cd "${TMPDIR}" || exit 1
tar -xf "${DESTFILE}"
cd "${DNAME}" || exit 1

find . -type f -print0 | xargs -0 file --mime-type | grep -E "application/x-(pie-executable|application|sharedlib)" | while read -r line ; do
    executable="$(echo "${line}" | cut -d ':' -f 1)"

    if ! patchelf --print-interpreter "${executable}" >/dev/null 2>&1 ; then
        continue
    fi

    interpreter="$(patchelf --print-interpreter "${executable}" | sed -e 's|^/lib64|/usr/lib|g')"
    echo "Fixing DT_INTERP in ${executable}"
    patchelf --set-interpreter "${interpreter}" "${executable}"
done

cd "${TMPDIR}" || exit 1

if [ "${ZEXT}" = "gz" ]; then
    ZTOOL="gzip"
else
    echo "*** Unknown compression extension: ${ZEXT}" >&2
    exit 1
fi

tar -cf - "${DNAME}" | ${ZTOOL} -9c > "${DESTFILE}"
