#!/bin/sh
#
# Walk the upstream file and create download URLs for additional
# crates we are including with the rust build.
#

PATH=/usr/bin
CWD="$(pwd)"

[ -f "${CWD}"/upstream ] || exit 0

crate=
ver=

while read -r line ; do
    # skip blank lines and comments
    if echo "${line}" | grep -q -E "(^$|^#)" >/dev/null 2>&1 ; then
        continue
    fi

    # pick up the crate name
    if echo "${line}" | grep -q -E "^\[.+\]$" >/dev/null 2>&1 ; then
        crate="$(echo "${line}" | sed -e 's|^\[||g' -e 's|\]$||g')"
        continue
    fi

    # pick up the version
    if echo "${line}" | grep -q -E "^ver:" >/dev/null 2>&1 ; then
        ver="$(echo "${line}" | cut -d ' ' -f 2)"
        continue
    fi

    # output a new URL when we encounter a gitrepo line
    if echo "${line}" | grep -q -E "^gitrepo:" >/dev/null 2>&1 ; then
        echo "https://crates.io/api/v1/crates/${crate}/${ver}/download@"
    fi
done < "${CWD}"/upstream

exit 0
