#!/bin/sh
#
# Find the bootstrap archive filenames that the Rust source expects
# to find cached.
#

PATH=/usr/bin
CWD="$(pwd)"
make clean extract

WRKSRC="$(make show var=WRKSRC)"

if [ -f "${WRKSRC}"/src/stage0 ]; then
    STAGE0_VER="$(grep ^compiler_version= "${WRKSRC}"/src/stage0 | cut -d '=' -f 2)"
elif [ -f "${WRKSRC}"/src/stage0.txt ]; then
    STAGE0_VER="$(grep ^rustc: "${WRKSRC}"/src/stage0.txt | cut -d ' ' -f 2)"
elif [ -f "${WRKSRC}"/src/stage0.json ]; then
    STAGE0_VER="$(grep version "${WRKSRC}"/src/stage0.json | awk '{ print $2; }' | sed -e 's|"||g')"
else
    echo "*** Unable to find the Stage 0 version" >&2
    exit 1
fi

if [ -z "${STAGE0_VER}" ]; then
    echo "*** Missing STAGE0_VER" >&2
    exit 1
fi

for mach in x86_64 powerpc64 ; do
    { echo "rust-std-${STAGE0_VER}-${mach}-unknown-linux-gnu.tar.gz"
      echo "rust-std-${STAGE0_VER}-${mach}-unknown-linux-gnu.tar.gz.asc"
      echo "rustc-${STAGE0_VER}-${mach}-unknown-linux-gnu.tar.gz"
      echo "rustc-${STAGE0_VER}-${mach}-unknown-linux-gnu.tar.gz.asc"
      echo "cargo-${STAGE0_VER}-${mach}-unknown-linux-gnu.tar.gz"
      echo "cargo-${STAGE0_VER}-${mach}-unknown-linux-gnu.tar.gz.asc"
      echo "clippy-${STAGE0_VER}-${mach}-unknown-linux-gnu.tar.gz"
      echo "clippy-${STAGE0_VER}-${mach}-unknown-linux-gnu.tar.gz.asc"
    } >> "${CWD}"/archives
done
