ghc-9.x can no longer compile itself with "make" and that won't be fixed upstream.  The
new system is called "hadrian", which uses the "shake" Haskell package underneath.  Hadrian
isn't a separately distributed thing.  It's part of the ghc source tree.

This document will explain how to move from 8.x built with make to 9.x built with hadrian.


=== PREP ===

* Remove all Haskell packages from your system.  Make sure /usr/lib/ghc* is removed as
  well.  I don't know why this persists as a skeleton directory tree.  We want to start
  with a system that is completely free of Haskell everything.

* Haskell requires Haskell to build, so we need to install a minimal environment.  This
  will also require certain older C libraries to be installed, since the environment will
  be linked against those.

    * Install the following libraries from Peachtree 3.6 packages into /usr/local/lib:
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/libffi-3.3-2.x86_64.dist

    * Install the following Haskell packages from Peachtree 3.6:
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/ghc-8.10.2-3.x86_64.dist
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/haskell-locale-8.10.2-6.x86_64.dist
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/haskell-types-8.10.2-8.x86_64.dist
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/blaze-8.10.2-5.x86_64.dist
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/data-default-8.10.2-6.x86_64.dist
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/haskell-regex-8.10.2-5.x86_64.dist
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/haskell-base-8.10.2-9.x86_64.dist
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/haskell-network-base-8.10.2-6.x86_64.dist
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/haskell-date-time-8.10.2-4.x86_64.dist
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/haskell-language-8.10.2-4.x86_64.dist
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/echo-0.1.3-5.x86_64.dist
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/asn1-8.10.2-6.x86_64.dist
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/haskell-archives-8.10.2-5.x86_64.dist
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/haskell-text-8.10.2-6.x86_64.dist
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/haskell-network-8.10.2-6.x86_64.dist
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/haskell-crypto-8.10.2-5.x86_64.dist
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/haskell-testing-8.10.2-5.x86_64.dist
      ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/cabal-install-3.2.0.0-2.x86_64.dist

* Install the following additional Haskell packages from Peachtree 3.6 that are necessary
  to build ghc:
  ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/alex-3.2.5-4.x86_64.dist
  ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/happy-1.20.0-1.x86_64.dist
  ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/hscolour-1.24.4-14.x86_64.dist
  ftp://ftp.burdell.org/pub/peachtree/3.6.0/dist/3.6.0/base/x86_64/c2hs-0.28.6-10.x86_64.dist


=== BUILD HASKELL PACKAGES, FIRST PASS ===

* If you haven't before, it's worth making sure cabal has the latest stuff.

    $ cabal update

* Build the ghc package as normal.  Part of this process will involve using cabal (which we
  previously installed above) to fetch dependencies for hadrian, most notably "extra" and
  "shake".  We will add these to a package later, but for now they'll just download into the
  ~/.cabal directory of whoever's building the package.

    $ cd ~/src/peachtree/base/haskell/ghc
    $ make world

* Build everything in BUILD-DEPENDENT-ORDER through cabal-install.  These tools are all necessary
  to build ghc and will be looked for as part of our standard configure args.  They should
  use the sysroot ghc (so, the new ghc package you just built) so they'll still work after
  removing the old environment.

* Clean up the old minimal Haskell environment we previously installed:

    * Remove all the Haskell packages we previously installed.  Again, you'll probably also
      have to manually remove /usr/lib/ghc*.

    * Remove /usr/local/lib/libffi*.  Nothing should be using it now.

    * Remove all the hadrian dependencies (extra and shake, etc.) that got installed to your
      ~/.cabal directory as part of building hadrian earlier.  Those will be linked against
      the 8.x ghc that no longer exists.  You can just rm -r ~/.cabal since nothing in there
      is usable anymore.

* Install the new packages you just built.  Be sure to do etcmerge on the following packages:
      ghc
      cabal-install
      xmonad
  At the very least for ghc so you get the new ld.so.conf.d file.


=== BUILD HASKELL PACKAGES, SECOND PASS ===

* Even though hadrian won't download anything with cabal, it's still going to use it to
  check.  So make sure you have a ~/.cabal/config file present and run "cabal update".
  There's probably some way to disable this, but it wasn't worth my time to figure out.

* You should now have everything necessary to build ghc using only our packages.  Go back
  and build everything in BUILD-DEPENDENT-ORDER from start to finish.  I like to install
  each package to the system as I go, just to make sure I have everything installed on
  the builder.
