# This Makefile is used as an interface between the Peachtree make-driven
# build system and the ghc Cabal-driven package system.  It presumes the
# existence of several environment variables.

ifndef TARGET_MACH
TARGET_MACH := $(shell uname -m | sed -e "s/i[[:digit:]]86/i386/g")
endif

# The ghc executable to use during package builds
GHC = $(SYSROOT)$(BINDIR)/ghc

# Get the version of ghc in sysroot
GHC_VER = $(shell env SYSROOT=$(SYSROOT) LD_LIBRARY_PATH=$(SYSROOT)$(LIBDIR) $(SYSROOT)$(LIBDIR)/ghc/bin/ghc --version | rev | cut -d ' ' -f 1 | rev)

# All the ghc stuff is installed under this directory
GHC_BASE_DIR = $(LIBDIR)/ghc-$(GHC_VER)
GHC_LIB_DIR = $(GHC_BASE_DIR)/lib/$(TARGET_MACH)-linux-ghc-$(GHC_VER)
GHC_PACKAGE_DIR = $(GHC_BASE_DIR)/lib/package.conf.d

# We need to set up a couple environment variables to trick ghc into
# running against a sysroot.  It really doesn't want to do that.
export GCONV_PATH := $(WRKSYSROOT)$(LIBDIR)/gconv

CONFIGURE_OPTS += --datasubdir=$(PKG) \
                  --package-db=clear \
                  --package-db=$(SYSROOT)$(GHC_PACKAGE_DIR) \
                  --ghc-option=-O2

all: build

mangle-package-cache:
	haveconf="$$(ls -1 $(SYSROOT)$(GHC_PACKAGE_DIR)/*conf 2>/dev/null)" ; \
	if [ ! -z "$${haveconf}" ]; then \
		find $(SYSROOT)$(GHC_PACKAGE_DIR) -type f -name "*.conf" | while read f ; do \
			sed -i -e 's|$${pkgroot}/..|$(SYSROOT)$(GHC_BASE_DIR)|g' \
			       -re "s|([ :\t])$(LIBDIR)|\1$(SYSROOT)$(LIBDIR)|g" \
			       -re "s|([ :\t])$(DATADIR)|\1$(SYSROOT)$(DATADIR)|g" $${f} ; \
		done ; \
		env SYSROOT=$(SYSROOT) LD_LIBRARY_PATH=$(SYSROOT)$(LIBDIR):$(GHC_LIB_DIR) $(SYSROOT)$(BINDIR)/ghc-pkg --package-db $(SYSROOT)$(GHC_PACKAGE_DIR) recache ; \
	fi

unmangle-package-cache:
	haveconf="$$(ls -1 $(DESTDIR)$(GHC_PACKAGE_DIR)/*conf 2>/dev/null)" ; \
	if [ ! -z "$${haveconf}" ]; then \
		find $(SYSROOT)$(GHC_PACKAGE_DIR) -type f -name "*.conf" | while read f ; do \
			sed -i -e 's|\"-Wl,--sysroot=.\+||g' -e "s|$(SYSROOT)||g" $${f} ; \
		done ; \
	fi

build: mangle-package-cache
ifeq ($(HS_MODULES),)
	if [ ! -f Setup.lhs ] && [ ! -f Setup.hs ]; then \
		echo "#!/usr/bin/env runhaskell" > Setup.lhs ; \
		echo "> module Main where" >> Setup.lhs ; \
		echo "> import Distribution.Simple" >> Setup.lhs ; \
		echo "> main :: IO ()" >> Setup.lhs ; \
		echo "> main = defaultMain" >> Setup.lhs ; \
	fi
	env $(CONFIGURE_ENV) LD_LIBRARY_PATH=$(WRKSYSROOT)$(GHC_LIB_DIR):$(LD_LIBRARY_PATH) runhaskell Setup configure $(CONFIGURE_OPTS) --ipid=$(PKG)-$(VER) --libdir=$(GHC_BASE_DIR)/lib --libsubdir=$(TARGET_MACH)-linux-ghc-$(GHC_VER)/$(PKG)-$(VER) --dynlibdir=$(GHC_LIB_DIR) --htmldir=$(DATADIR)/doc/ghc-$(GHC_VER)/html/libraries/$(PKG)-$(VER)
	env $(BUILD_ENV) LD_LIBRARY_PATH=$(WRKSYSROOT)$(GHC_LIB_DIR):$(LD_LIBRARY_PATH) runhaskell Setup build
	env $(BUILD_ENV) runhaskell Setup haddock
else
	for hsmodule in $(HS_MODULES) ; do \
		( cd $(WRKSRC)/$$hsmodule ; \
		  if [ ! -f Setup.lhs ] && [ ! -f Setup.hs ]; then \
		      echo "#!/usr/bin/env runhaskell" > Setup.lhs ; \
		      echo "> module Main where" >> Setup.lhs ; \
		      echo "> import Distribution.Simple" >> Setup.lhs ; \
		      echo "> main :: IO ()" >> Setup.lhs ; \
		      echo "> main = defaultMain" >> Setup.lhs ; \
		  fi ; \
		  env $(CONFIGURE_ENV) LD_LIBRARY_PATH=$(WRKSYSROOT)$(GHC_LIB_DIR):$(LD_LIBRARY_PATH) runhaskell Setup configure $(CONFIGURE_OPTS) --ipid=$$hsmodule --libdir=$(GHC_BASE_DIR)/lib --libsubdir=$(TARGET_MACH)-linux-ghc-$(GHC_VER)/$$hsmodule --dynlibdir=$(GHC_LIB_DIR) --htmldir=$(DATADIR)/doc/ghc-$(GHC_VER)/html/libraries/$$hsmodule ; \
		  env $(BUILD_ENV) LD_LIBRARY_PATH=$(WRKSYSROOT)$(GHC_LIB_DIR):$(LD_LIBRARY_PATH) runhaskell Setup build ; \
		  env $(BUILD_ENV) runhaskell Setup haddock ; \
		  env $(BUILD_ENV) runhaskell Setup copy --destdir=$(SYSROOT) ; \
		  env $(BUILD_ENV) runhaskell Setup register --gen-pkg-config=$(SYSROOT)$(GHC_PACKAGE_DIR)/$$hsmodule.conf ; \
		  env $(BUILD_ENV) ghc-pkg recache --package-db=$(SYSROOT)$(GHC_PACKAGE_DIR) ; \
		) ; \
		$(MAKE) mangle-package-cache ; \
	done
endif

install:
	mkdir -p $(DESTDIR)$(GHC_BASE_DIR)/lib/package.conf.d
ifeq ($(HS_MODULES),)
	env $(FAKEINST_ENV) runhaskell Setup copy --destdir=$(DESTDIR)
	env $(FAKEINST_ENV) runhaskell Setup register --gen-pkg-config=$(DESTDIR)$(GHC_PACKAGE_DIR)/$(PKG)-$(VER).conf
else
	for hsmodule in $(HS_MODULES) ; do \
		( cd $(WRKSRC)/$$hsmodule ; \
		  env $(FAKEINST_ENV) runhaskell Setup copy --destdir=$(DESTDIR) ; \
		  env $(FAKEINST_ENV) runhaskell Setup register --gen-pkg-config=$(DESTDIR)$(GHC_PACKAGE_DIR)/$$hsmodule.conf ; \
		) ; \
	done
endif
	$(MAKE) unmangle-package-cache
