{-# LANGUAGE LambdaCase #-}
-- Replacement for the default Mod+Shift+q operation in Xmonad
-- Presents a dialog box asking the user to choose between shutdown, restart,
-- suspend, or logout.
--
-- Copyright (C) 2013-2016 Chris Lumens <chris@bangmoney.org>
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.
import           Control.Monad(void)
import           Data.Text(pack)
import qualified GI.Gtk.Functions as Gtk
import           GI.Gtk.Objects.Box(boxSetSpacing, boxPackStart)
import           GI.Gtk.Objects.Dialog(dialogAddButton, dialogGetContentArea, dialogNew, dialogRun)
import           GI.Gtk.Objects.Label(labelNew)
import           GI.Gtk.Objects.Widget(widgetDestroy, widgetShowAll)
import           GI.Gtk.Objects.Window(windowSetModal)
import           System.Exit(ExitCode(..), exitWith)
import           System.Process(callProcess)

main :: IO ()
main = do
    void $ Gtk.init Nothing

    lbl <- labelNew $ Just (pack "End Session?")

    dlg <- dialogNew
    box <- dialogGetContentArea dlg
    boxSetSpacing box 6
    boxPackStart box lbl False False 0

    mapM_ (\(s, r) -> void $ dialogAddButton dlg (pack s) r)
          [("_Logout", 0), ("_Reboot", 1), ("_Shutdown", 2), ("_Cancel", 3)]

    windowSetModal dlg True
    widgetShowAll dlg

    dialogRun dlg >>= \case
        0 -> widgetDestroy dlg >> callProcess "/usr/bin/pkill" ["-KILL", "xmonad"]
        1 -> widgetDestroy dlg >> callProcess "/usr/bin/sudo" ["/usr/sbin/shutdown", "-r", "now"]
        2 -> widgetDestroy dlg >> callProcess "/usr/bin/sudo" ["/usr/sbin/shutdown", "-P", "-h", "now"]
        3 -> widgetDestroy dlg >> exitWith ExitSuccess
