# Create directory symlinks that do not include version numbers so our
# docs and licenses trees are constructed correctly.
post-patch:
	for module in $(HS_MODULES) ; do \
		modulename="$$(echo "$$module" | sed -E 's|\-[0-9\.]+$$||g')" ; \
		ln -s $$module $(WRKSRC)/$$modulename ; \
	done

post-fakeinst:
	# Make sure the package.conf.d files have sysroot references removed
	find $(WRKINST)$(LIBDIR)/ghc-$(GHC_VER)/lib/package.conf.d -type f -name "*.conf" | while read f ; do \
		sed -i -e 's|$(WRKSYSROOT)||g' $$f ; \
	done

	# Set permissions on documentation.
	if [ -d $(WRKINST)$(DOCDIR) ]; then \
		find $(WRKINST)$(DOCDIR)/ -type f | while read -r f ; do \
			chmod 0644 $$f ; \
		done \
	fi

	# Remove extra license files cabal installs
	if [ -d $(WRKINST)$(DATADIR)/haskell/doc ]; then \
		rm -r $(WRKINST)$(DATADIR)/haskell/doc ; \
		rmdir --ignore-fail-on-non-empty $(WRKINST)$(DATADIR)/haskell ; \
	fi

# Do some clever renaming of the patch files.  Because the WRKSRC variable is
# set to match WRKDIR, the update-patches target will work a little strangely.
# We were also storing the patch files here without the Hackage module
# subdirectory.  That way each time we update this package we don't have to
# add and remove the same patch over and over again because the version
# number in the directory name changes.
#
# This target will trim the WRKSRC and Hackage module subdirectory from the
# patch name and move it in to the place we expect.  Gotta love shell.
post-update-patches:
	for p in $(CWD)/patches/*.patch ; do \
		patchfile="$$(basename $$p)" ; \
		suffix="$$(head -n 2 $$p | tail -n 1 | cut -f 1 | cut -c5-)" ; \
		badpatch="$$(echo "$(WRKSRC)$$suffix" | sed -e 's|/|_|g').patch" ; \
		i=$$(($$(expr index "$$suffix" '/') + 1)) ; \
		newpatch="$$(echo "$$suffix" | cut -c$${i}- | sed -e 's|/|_|g').patch" ; \
		if [ -f "$(CWD)/patches/$$badpatch" ]; then \
			mv "$(CWD)/patches/$$badpatch" "$(CWD)/patches/$$newpatch" ; \
		fi ; \
	done
