#!/bin/sh -x
#
# Update /etc/resolv.conf based on DNS settings sent over in the
# OpenVPN link.  Parses foreign_option_* environment variables and
# looks for "dhcp-option" settings that contain DNS information.
# Calls the resolvconf program to do the actual file updating.
#
# Author:  David Cantrell <dcantrell@burdell.org>
#
# Example envs set from openvpn:
# foreign_option_1='dhcp-option DNS 193.43.27.132'
# foreign_option_2='dhcp-option DNS 193.43.27.133'
# foreign_option_3='dhcp-option DOMAIN be.bnc.ch'
# foreign_option_4='dhcp-option DOMAIN-SEARCH bnc.local'

PATH=/usr/bin
CWD="$(pwd)"
RESOLVCONF="/usr/sbin/resolvconf"
TMPAREA="$(mktemp -d)"
trap 'rm -rf "${TMPAREA}"' EXIT

if [ "${script_type}" = "up" ]; then
    env | grep "^foreign_option_" | while read var ; do
        optname="$(echo "${var}" | cut -d '=' -f 1)"
        val="$(eval echo "\$${optname}")"

        echo ${val} | while read option name value ; do
            [ "${option}" = "dhcp-option" ] || continue
            if [ "${name}" = "DNS" ]; then
                echo "==> DNS nameserver for VPN found: ${value}"
                echo "${value}" >> ${TMPAREA}/nameservers
            elif [ "${name}" = "DOMAIN" ] || [ "${name}" = "DOMAIN-SEARCH" ]; then
                echo "==> DNS domain for VPN found: ${value}"
                echo "${value}" >> ${TMPAREA}/search
            fi
        done
    done

    if [ -s "${TMPAREA}/search" ]; then
        echo "search $(paste -s -d ' ' ${TMPAREA}/search)" >> ${TMPAREA}/changes
    fi

    if [ -s "${TMPAREA}/nameservers" ]; then
        sed -e 's|^|nameserver |g' < ${TMPAREA}/nameservers >> ${TMPAREA}/changes
    fi

    if [ -s "${TMPAREA}/changes" ]; then
        cat ${TMPAREA}/changes | ${RESOLVCONF} -x -a ${dev}.inet
    fi
elif [ "${script_type}" = "down" ]; then
    ${RESOLVCONF} -d ${dev}.inet
fi
