#!/bin/sh
#
# Given a Thunderbird release version number, find the hg repo tag
# that maps to it.
# by: David Cantrell <dcantrell@burdell.org>
#

PATH=/usr/bin
CWD="$(pwd)"
PROGNAME="$(basename "$0")"
VER="$1"

if [ -z "${VER}" ]; then
    echo "Usage: ${PROGNAME} [version]" >&2
    exit 1
fi

# This is a repo containing all of the released versions of Thunderbird.
# Actual development happens here:
#     https://hg.mozilla.org/comm-central/
# But we pull from the repo that contains release tags.
REPO="https://hg-edge.mozilla.org/comm-unified"

# Released versions are tagged as "THUNDERBIRD_VERSIONesr_RELEASE"
# where VERSION is of the form X for the version X.Y.Z.  The "esr"
# part is always present, until they change their minds.
TAG="THUNDERBIRD_$(echo "${VER}" | tr '.' '_')_RELEASE"

# Function to get the commit ID matching the TAG we want.
get_ID() {
    found=0

    curl -s "${REPO}"/tags | while read -r line ; do
        if [ ${found} -eq 0 ]; then
            if echo "${line}" | grep -q "${TAG}" >/dev/null 2>&1 ; then
                found=1
            fi
        elif [ ${found} -eq 1 ]; then
            if echo "${line}" | grep -q '>files</a>' >/dev/null 2>&1 ; then
                echo "${line}" | cut -d '"' -f 2 | rev | cut -d '/' -f 1 | rev
                break
            fi
        fi
    done
}

ID="$(get_ID)"

if [ -z "${ID}" ]; then
    echo "*** Unable to find commit ID for tag ${TAG}" >&2
    exit 1
fi

echo "${ID}"
