/*
 * Copyright David Cantrell <dcantrell@burdell.org>
 * SPDX-License-Identifier: Apache-2.0
 */

#ifndef _PLIST_H
#define _PLIST_H

#include <sys/queue.h>

/*
 * This describes an entry from a plist file (or attrs file in a .dist).
 * We are not using struct finfo because plist does not represent
 * everything that finfo needs.  The plist file just provides the name,
 * owner, group, and mode.  Everything else is gathered up by the stat(2)
 * used to populate a finfo.
 */
typedef struct _plist_entry_t {
    char *filename;
    char *owner;
    char *group;
    int64_t uid;
    int64_t gid;
    mode_t mode;
    bool is_symlink;
    bool has_checksum;
    TAILQ_ENTRY(_plist_entry_t) items;
} plist_entry_t;

/*
 * Linked list defining an explanded plist file for a package.
 */
typedef TAILQ_HEAD(plist_s, _plist_entry_t) plist_t;

#endif /* _PLIST_H */
