/* Check a package file for a valid signature. */

/*
 * Copyright David Cantrell <dcantrell@burdell.org>
 * Copyright David Shea <david@reallylongword.org>
 * SPDX-License-Identifier: Apache-2.0
 */

#include <stdlib.h>
#include <stdbool.h>
#include <assert.h>
#include <string.h>
#include "dist-tools.h"

/*
 * A dist package will have a file in the archive named 'signature'.  It
 * will contain the text defined in SIGNATURE.  To verify a file is a
 * package, we check for the signature in the archive.  Not foolproof, but
 * ok for our purposes.
 */
bool checksig(char *filename)
{
    size_t prefixlen = strlen(SIGNATURE);
    size_t verlen = strlen(SIGNATURE_IN_USE);
    char *buf = NULL;
    bool have_signature = true;
    bool valid_signature = false;

    assert(filename != NULL);

    buf = calloc(1, BUFSIZ);
    assert(buf != NULL);

    if (unpack_metadata2str(filename, DIST_SIGNATURE, buf, true)) {
        return false;
    }

    /* trim trailing newline and/or carriage return characters */
    buf[strcspn(buf, "\r\n")] = '\0';

    if (strlen(buf) > (prefixlen + verlen)) {
        free(buf);
        return false;
    }

    /* Check for valid signature prefix followed by a valid version. */
    if (!strncmp(buf, SIGNATURE, prefixlen) && (!strncmp(buf+prefixlen, SIGNATURE_21, verlen) || !strncmp(buf+prefixlen, SIGNATURE_22, verlen))) {
        valid_signature = true;
    }

    free(buf);

    if (have_signature && valid_signature) {
        return true;
    } else {
        return false;
    }
}
