/* Initialize the installed packages database. */

/*
 * Copyright David Cantrell <dcantrell@burdell.org>
 * SPDX-License-Identifier: Apache-2.0
 */

#include <stdlib.h>
#include <assert.h>
#include <err.h>
#include <string.h>
#include "dist-tools.h"
#include "db.h"

/* Initialize the installed packages database if we need to. */
int initdb(char *prepend, const char *dbpath, const char *dbfile)
{
    char *errmsg = NULL;
    char *tmp = NULL;
    sqlite3 *db = NULL;

    assert(dbpath != NULL);
    assert(dbfile != NULL);

    /* create the installed package database directory if we need to */
    tmp = joinpath(prepend, dbpath, NULL);

    if (mkdirp(tmp, _DIR_MODE)) {
        warn("mkdirp");
        free(tmp);
        return -1;
    }

    /* path to the database file */
    free(tmp);
    tmp = joinpath(prepend, dbfile, NULL);

    /* open database */
    db = open_db(tmp);

    if (db == NULL) {
        free(tmp);
        return -1;
    }

    free(tmp);

    /* initialize database */
    if (sqlite3_exec(db, DB_PACKAGES_SCHEMA, NULL, 0, &errmsg) != SQLITE_OK) {
        warn("%s (%d)", sqlite3_errmsg(db), sqlite3_errcode(db));
        sqlite3_free(errmsg);
        return -1;
    }

    if (sqlite3_exec(db, DB_CONTENTS_SCHEMA, NULL, 0, &errmsg) != SQLITE_OK) {
        warn("%s (%d)", sqlite3_errmsg(db), sqlite3_errcode(db));
        sqlite3_free(errmsg);
        return -1;
    }

    /* we are finished with the databases here */
    sqlite3_free(errmsg);
    sqlite3_close(db);

    return 0;
}
