/*
 * Makes sure the file is a valid ELF file and is a shared object or an
 * executable.
 */

/*
 * Copyright David Cantrell <dcantrell@burdell.org>
 * Copyright David Shea <david@reallylongword.org>
 * Copyright Chris Lumens <chris@bangmoney.org>
 * SPDX-License-Identifier: Apache-2.0
 */

#include <stdlib.h>
#include <unistd.h>
#include <stdbool.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <limits.h>
#include <err.h>
#include <libelf.h>
#include "dist-tools.h"

bool is_elf(const char *filename)
{
    int fd;
    bool ret = false;
    Elf *elf = NULL;
    static bool initialized = false;

    if (filename == NULL) {
        return false;
    }

    if (!initialized) {
        if (elf_version(EV_CURRENT) == EV_NONE) {
            warn("elf_version");
            return false;
        }

        initialized = true;
    }

    if ((fd = open(filename, O_RDONLY | O_CLOEXEC)) == -1) {
        warn("open");
        return false;
    }

    elf = elf_begin(fd, ELF_C_READ_MMAP_PRIVATE, NULL);

    if (elf_kind(elf) == ELF_K_ELF) {
        ret = true;
    }

    if (elf_end(elf) != 0) {
        warn("elf_end");
        return false;
    }

    if (close(fd) == -1) {
        warn("close");
        return false;
    }

    return ret;
}
