/*
 * Perform a 'mkdir -p' operation on a given path.
 */

/*
 * Copyright David Cantrell <dcantrell@burdell.org>
 * SPDX-License-Identifier: Apache-2.0
 */

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <sys/stat.h>
#include <sys/types.h>

int mkdirp(char *path, mode_t mode)
{
    int r = 0;
    char *p = NULL;
    char *start = NULL;
    struct stat sb;

    assert(path != NULL);
    assert(*path == '/');

    start = p = strdup(path);
    p++;

    /* handle all the parent directories */
    while (*p != '\0') {
        if (*p == '/') {
            *p = '\0';
            r = stat(start, &sb);

            if (r == 0 && S_ISDIR(sb.st_mode)) {
                *p = '/';
                p++;
                continue;
            } else if (mkdir(start, mode) == -1) {
                return -1;
            }

            *p = '/';
        }

        p++;
    }

    /* final directory */
    if ((stat(start, &sb) != 0) && (mkdir(start, mode) == -1)) {
        return -1;
    }

    free(start);

    return 0;
}
