/* Parsing support functions. */

/*
 * Copyright David Cantrell <dcantrell@burdell.org>
 * SPDX-License-Identifier: Apache-2.0
 */

#include <stdint.h>
#include <string.h>
#include <assert.h>
#include <errno.h>
#include <err.h>
#include <limits.h>
#include "dist-tools.h"

/*
 * Split plist owner and group fields in to the name and id.
 * Input is a string of the form:  name(ID)
 * where name is a text string and ID is the user or group ID number.
 * The parentheses are for human readability, so we parse those out.
 */
int split_name_id(const char *field, char **name, uint64_t *id)
{
    char *orig = NULL;
    char *input = NULL;
    char **tokens = NULL;

    assert(field != NULL);
    assert(name != NULL);

    input = strdup(field);
    orig = input;
    tokens = strsplit(input, "()");

    if (tokens == NULL) {
        warn("unable to split name id %s", field);
        return -1;
    }

    if (strvlen(tokens) != 3) {
        warn("invalid name(ID) token: %s", field);
        strvfree(tokens);
        free(orig);
        return -1;
    }

    *name = strdup(tokens[0]);
    assert(*name != NULL);

    errno = 0;
    *id = strtoull(tokens[1], NULL, 10);

    if ((errno == ERANGE && *id == ULLONG_MAX) || (errno == EINVAL && *id == 0)) {
        warn("invalid ID: %s", tokens[1]);
        strvfree(tokens);
        free(orig);
        return -1;
    }

    strvfree(tokens);
    free(orig);

    return 0;
}
