/* Remove package entry from the database. */

/*
 * Copyright David Cantrell <dcantrell@burdell.org>
 * SPDX-License-Identifier: Apache-2.0
 */

#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <err.h>
#include "dist-tools.h"
#include "db.h"

/*
 * Given a struct dist, remove its entry from the package database
 * and all files that belong to its package ID.
 */
int rmentry(const struct dist *package, const char *db_installed)
{
    char *errmsg = NULL;
    char *cmd = NULL;
    sqlite3 *db = NULL;

    assert(package != NULL);
    assert(package->id >= 0);

    /* open the installed package database */
    db = open_db(db_installed);

    if (db == NULL) {
        return -1;
    }

    /* remove the entry from the packages table */
    xasprintf(&cmd, "DELETE FROM packages WHERE id='%d'", package->id);

    if (sqlite3_exec(db, cmd, NULL, 0, &errmsg) != SQLITE_OK) {
        warnx("sqlite3_exec: %s", errmsg);
        sqlite3_free(errmsg);
        return -1;
    }

    free(cmd);
    cmd = NULL;

    /* remove the entries from the contents table */
    xasprintf(&cmd, "DELETE FROM contents WHERE packageid='%d'", package->id);

    if (sqlite3_exec(db, cmd, NULL, 0, &errmsg) != SQLITE_OK) {
        warnx("sqlite3_exec: %s", errmsg);
        sqlite3_free(errmsg);
        return -1;
    }

    free(cmd);
    cmd = NULL;

    /* clean up */
    sqlite3_free(errmsg);
    sqlite3_close(db);

    return 0;
}
