/* Recursively removes a directory. */

/*
 * Copyright David Shea <david@reallylongword.org>
 * Copyright David Cantrell <dcantrell@burdell.org>
 * SPDX-License-Identifier: Apache-2.0
 */

#include <stdbool.h>
#include <unistd.h>
#include <string.h>
#include <ftw.h>
#include <err.h>
#include "dist-tools.h"

/* Local prototypes */
static int _rmtree_entry(const char *, const struct stat *, int, struct FTW *);

static int _rmtree_entry(const char *fpath, __attribute__((unused)) const struct stat *sb, __attribute__((unused)) int tflag, __attribute__((unused)) struct FTW *ftwbuf)
{
    return remove(fpath);
}

/* Remove specified tree, ignoring errors if specified. */
int rmtree(const char *path, const bool ignore_errors, const bool contentsonly)
{
    struct stat sb;
    int status = 0;
    int flags = FTW_DEPTH | FTW_MOUNT | FTW_PHYS;

    if (path == NULL) {
        return 0;
    }

    if (stat(path, &sb) == -1) {
        if (ignore_errors) {
            return 0;
        } else {
            warn("stat");
            return 1;
        }
    }

    if (!S_ISDIR(sb.st_mode)) {
        if (ignore_errors) {
            return 0;
        } else {
            warnx("%s: not a directory", path);
            return 2;
        }
    }

    status = nftw(path, _rmtree_entry, 25, flags);

    if (contentsonly) {
        return status;
    } else {
        return (status && rmdir(path));
    }
}
