/*
 * Split a string in to an array of substrings, using the string of
 * delimiters as the breaking points.
 */

/*
 * Copyright David Cantrell <dcantrell@burdell.org>
 * SPDX-License-Identifier: Apache-2.0
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <assert.h>
#include "dist-tools.h"

/*
 * Tokenize a string based on tokens listed in delim, return the tokens
 * as a char array.  The caller needs to free the char array because it
 * will contain pointers to copies of the passed in string.  This is
 * because the strsep() function has to modify the original string.
 */
char **strsplit(char *s, char *delim)
{
    int i = 0;
    char *token = NULL;
    char *walk = s;
    char *head = NULL;
    char **ret = NULL;

    assert(s != NULL);
    assert(delim != NULL);

    /* compute length of ret array */
    while (*walk++ != '\0') {
        if (index(delim, *walk) != NULL) {
            i++;
        }
    }

    /* add one morefor theNULL sentinel */
    i++;

    /* allocate return array */
    ret = calloc(i, sizeof(char *));
    assert(ret != NULL);

    /* split up the string */
    head = walk = strdup(s);
    i = 0;

    while ((token = strsep(&walk, delim)) != NULL) {
        ret[i] = strdup(token);
        i++;
    }

    ret[i] = NULL;
    free(head);
    return ret;
}

/*
 * Return number of elements in an array of strings.
 */
int strvlen(char **s)
{
    int len = 0;

    if (s == NULL) {
        return len;
    }

    while (*s++ != NULL) {
        len++;
    }

    return len;
}

/*
 * Free an array of strings.  Assumes the array elements must be freed.
 */
void strvfree(char **s)
{
    int i = 0;

    while (s[i] != NULL) {
        free(s[i]);
        i++;
    }

    free(s);
    return;
}
