/*
 * Trims leading and trailing whitespace from a dynamically allocated
 * string.  NOTE:  Must call this on a string that has been malloc'ed
 * so free() can be called.  This function uses memmove() and realloc().
 */

/*
 * Copyright David Cantrell <dcantrell@burdell.org>
 * SPDX-License-Identifier: Apache-2.0
 */

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>

/*
 * Trim leading and trailing whitespace from a string.
 */
char *strtrim(const char *s)
{
    char *r = NULL;
    char *tmp = NULL;
    char *start = NULL;
    int i = 0;

    assert(s != NULL);
    start = tmp = strdup(s);
    assert(tmp != NULL);

    /* trim leading whitespace */
    while (isspace(*start)) {
        start++;
    }

    /* length of remaining string */
    i = strlen(start) - 1;

    /* trim trailing whitespace */
    while (i--) {
        if (isspace(start[i])) {
            start[i] = '\0';
        } else {
            break;
        }
    }

    r = strdup(start);
    assert(r != NULL);
    free(tmp);

    return r;
}
