/* Check a string to see if it's a valid architecture name. */

/*
 * Copyright David Cantrell <dcantrell@burdell.org>
 * SPDX-License-Identifier: Apache-2.0
 */

#include <err.h>
#include <string.h>
#include <sys/utsname.h>
#include "dist-tools.h"

/*
 * Target architecture is limited to what we get from uname() or noarch.
 */
int check_targetarch(char *a)
{
    struct utsname utsbuf;

    if (uname(&utsbuf)) {
        warn("uname");
        return -1;
    }

    if (!strcmp(a, utsbuf.machine) || !strcmp(a, "noarch")) {
        return 0;
    } else {
        return 1;
    }
}

/*
 * Return the default target architecture.  Caller must free returned string.
 */
char *getdefaultarch(void)
{
    struct utsname utsbuf;

    if (uname(&utsbuf)) {
        warn("uname");
        return NULL;
    }

    return strdup(utsbuf.machine);
}
