#!/bin/sh
#
# cleansum.sh: Reset the archives file to just have archive filenames
#              and include lines.
#
# Copyright David Cantrell <dcantrell@burdell.org>
# SPDX-License-Identifier: Apache-2.0

# This script returns zero on success, non-zero on error.
# This script is called by the build system scripts and should not be called
# directly.

if [ -z "${WRKDIR}" ] || [ ! -d "${WRKDIR}" ]; then
    exit 1
fi

CWD="$(pwd)"
NEW="${WRKDIR}/archives.new"

if [ -f "${CWD}/archives" ]; then
    rm -f "${NEW}"

    while read -r line ; do
        if echo "${line}" | grep -qE "\(.+\)=" >/dev/null 2>&1 ; then
            echo "${line}" | cut -d '(' -f 2 | cut -d ')' -f 1 >> "${NEW}"
            continue
        fi

        if echo "${line}" | grep -qE "^(SHA|MD)[0-9]+\ " >/dev/null 2>&1 ; then
            echo "${line}" | cut -d ' ' -f 3 >> "${NEW}"
            continue
        fi

        echo "${line}" >> "${NEW}"
    done < "${CWD}/archives"

    if [ -f "${NEW}" ]; then
        cp "${NEW}" "${CWD}/archives"
    fi
fi
