#!/bin/sh
#
# patch.sh: Apply patches to extracted source tree.
#
# Copyright David Cantrell <dcantrell@burdell.org>
# Copyright Chris Lumens <chris@bangmoney.org>
# SPDX-License-Identifier: Apache-2.0

# This script returns zero on success, non-zero on error.
# The following environment variables need to be set for the script:
#    $WRKDIR               The top level work directory.
#    $WRKSRC               The extracted source directory.
#    $PATCHLEVEL           The directory levels to trim when applying.
# This script is called by the build system scripts and should not be called
# directly.

if [ -z "${WRKSRC}" ] || [ ! -d "${WRKSRC}" ]; then
    exit 0
fi

CWD="$(pwd)"

( if [ -d "${CWD}/patches" ]; then
      find "${CWD}"/patches -type f -name "*.patch" | while read -r patchfile ; do
          patch -d "${WRKSRC}" -p "${PATCHLEVEL}" -F 0 -b -z .orig --verbose -i "${patchfile}"
      done
  fi
) 2>&1 | tee "${WRKDIR}"/patch.log
