#!/bin/sh
#
# update-plist.sh: Create or update package packing list.
#
# Copyright David Cantrell <dcantrell@burdell.org>
# SPDX-License-Identifier: Apache-2.0

# This script returns zero on success, non-zero on error.
# The following environment variables need to be set for the script:
#    $WRKDIR               The top level work directory.
#    $WRKINST              The package staging root directory.
#    $PACKAGE              The full package name.
#    $VER                  The package version.
#    $REL                  The package release.
#    $TARGET_NAME          The target arch name (e.g., vax-peachtree-linux).
#    $TARGET_MACH          arch(1) or 'uname -m' value (or 'noarch').
# This script is called by the build system scripts and should not be called
# directly.

if [ -z "${WRKDIR}" ] || [ ! -d "${WRKDIR}" ]; then
    exit 1
fi

if [ -z "${WRKINST}" ] || [ ! -d "${WRKINST}" ]; then
    exit 2
fi

[ -z "${PACKAGE}" ] && exit 3
[ -z "${VER}" ] && exit 4
[ -z "${REL}" ] && exit 5

CWD="$(pwd)"

if [ -d "${WRKINST}" ]; then
    printf ">>> Generating packing list for %s..." "${PACKAGE}"
    distplist -u root -g root -t "${WRKINST}" | sort -u > "${WRKDIR}"/plist.NEW

    if [ ! "${TARGET_MACH}" = "noarch" ]; then
        sed -e "s|${TARGET_NAME}|\${TARGET_NAME}|g" "${WRKDIR}"/plist.NEW | \
        sed -e "s|${TARGET_MACH}|\${TARGET_MACH}|g" | \
        sed -e "s|$(echo "${VER}" | sed -e 's|\.|\\.|g')-${REL}|\${VER}-\${REL}|g" | \
        sed -e "s|$(echo "${VER}" | sed -e 's|\.|\\.|g')|\${VER}|g" | \
        sort -u > "${WRKDIR}"/plist.NEW2
        mv "${WRKDIR}"/plist.NEW2 "${WRKDIR}"/plist.NEW
    fi

    if [ -f "${CWD}"/plist ]; then
        if diff -u "${CWD}"/plist "${WRKDIR}"/plist.NEW > "${WRKDIR}"/plist.changes 2>/dev/null ; then
            echo "no changes found."
            rm -f "${WRKDIR}"/plist.NEW
        else
            echo "CHANGES FOUND."
            cat "${WRKDIR}"/plist.changes
            rm -f "${WRKDIR}"/plist.changes
        fi
    else
        echo "done."
    fi

    if [ -f "${WRKDIR}"/plist.NEW ]; then
        mv "${WRKDIR}"/plist.NEW "${CWD}"/plist
    fi
fi
