#!/bin/sh
# Verify distadd checks ELF shared dependencies.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
PKG="test-1.0-1.$(uname -m).dist"

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"

# create the test package
# create package helper
create_pkg() {
    if [ -z "${LEAKCHECK}" ]; then
        ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" >/dev/null 2>&1
    else
        ${LEAKCHECK} ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}"
    fi
}

create_pkg

# install package helper
install_pkg() {
    [ -z "${LEAKCHECK}" ] && env ETCMERGE=${ETCMERGE} ${DISTADD} $* || env ETCMERGE=${ETCMERGE} ${LEAKCHECK} ${DISTADD} $*
}

# distadd deps check (missing deps)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r/etc
touch ${OUTPUT}/r/etc/ld.so.conf
/sbin/ldconfig -r ${OUTPUT}/r

echo -n "checking distadd ELF dependency checking (missing deps)..."
out="$(install_pkg -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
r=$?
out="$(echo "${out}" | grep -E " \*\*\* lib.* not found")"
if [ ${r} -eq 1 ] && [ ! -z "${out}" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi

# distadd deps check skip
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking distadd ELF dependency checking (skip depcheck)..."
out="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
r=$?
out="$(echo "${out}" | grep -E "^\*\*\* lib.* not found")"
if [ ${r} -eq 0 ] && [ -z "${out}" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected zero failures
[ ${FAILS} -gt 0 ] && exit 1 || exit 0
