#!/bin/sh
# Verify 'distadd -g' functionality works.

CWD="$(pwd)"

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
EXPECTED_FAILS=0

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk
( cd ${TREE}/usr/sbin ; ln -s distmk mkdist )

# create plist
u="$(id -un)($(id -u))"
g="$(id -gn)($(id -g))"
echo "usr $u $g 0755" > ${META}/plist
echo "usr/sbin $u $g 0755" >> ${META}/plist
echo "usr/sbin/distmk $u $g 0755" >> ${META}/plist
echo "usr/sbin/mkdist $u $g symlink(distmk)" >> ${META}/plist

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"
PKG="test-1.0-1.$(uname -m).dist"

# create the package
create_pkg() {
    if [ -z "${LEAKCHECK}" ]; then
        ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" >/dev/null 2>&1
    else
        ${LEAKCHECK} ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}"
    fi
}

create_pkg

# install package helper
install_pkg() {
    [ -z "${LEAKCHECK}" ] && env ETCMERGE=${ETCMERGE} ${DISTADD} $* || env ETCMERGE=${ETCMERGE} ${LEAKCHECK} ${DISTADD} $*
}

# verify functionality
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking 'distadd -g PKG' generates attributes..."
install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} >/dev/null 2>&1
install_pkg -r ${OUTPUT}/r test >/dev/null 2>&1
r1=$?
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
install_pkg -d -g -r ${OUTPUT}/r ${OUTPUT}/${PKG} >/dev/null 2>&1
install_pkg -r ${OUTPUT}/r test >/dev/null 2>&1
r2=$?
if [ ${r1} -ne ${r2} ]; then
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
else
    echo "passed."
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected $EXPECTED_FAILS failures
[ ${FAILS} -eq ${EXPECTED_FAILS} ] && exit 0 || exit 1
