#!/bin/sh
# Verify distadd can handle hard links.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
PKG1="test-1.0-1.$(uname -m).dist"
PKG2="test-2.0-1.$(uname -m).dist"

# create package helper
create_pkg() {
    if [ -z "${LEAKCHECK}" ]; then
        ${DISTMK} -t ${TREE} -p "test" -v "$1" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" >/dev/null 2>&1
    else
        ${LEAKCHECK} ${DISTMK} -t ${TREE} -p "test" -v "$1" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}"
    fi
}

# install package helper
install_pkg() {
    [ -z "${LEAKCHECK}" ] && env ETCMERGE=${ETCMERGE} ${DISTADD} $* || env ETCMERGE=${ETCMERGE} ${LEAKCHECK} ${DISTADD} $*
}

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp ${DISTMK} ${TREE}/usr/sbin/distmk
( cd ${TREE}/usr/sbin ; ln distmk distmk2 )

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
usr/sbin/distmk2 root(0) root(0) 0755
EOF

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"

create_pkg "1.0"

# make a variant package with different file symlinks
( cd ${TREE}/usr/sbin ; rm -f distmk2 )
( cd ${TREE}/usr/sbin ; ln distmk distmk3 )
( cd ${TREE}/usr/sbin ; ln distmk3 distmk2 )
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
usr/sbin/distmk2 root(0) root(0) 0755
usr/sbin/distmk3 root(0) root(0) 0755
EOF

create_pkg "2.0"

# install package with hard links
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking 'distadd' on package with hard links..."
out="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
r=$?
echo "${out}" | grep -q "Installed package $(basename ${PKG1} .dist) successfully" >/dev/null 2>&1
q=$?
if [ ${r} -eq 0 ] && [ ${q} -eq 0 ] && [ -x ${OUTPUT}/r/usr/sbin/distmk ] && [ -x ${OUTPUT}/r/usr/sbin/distmk2 ] && [ $(stat -c %h ${OUTPUT}/r/usr/sbin/distmk) -eq 2 ] && [ $(stat -c %h ${OUTPUT}/r/usr/sbin/distmk2) -eq 2 ] && [ "$(stat -c %i ${OUTPUT}/r/usr/sbin/distmk)" = "$(stat -c %i ${OUTPUT}/r/usr/sbin/distmk2)" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# upgrade a package with hard links
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking 'distadd -u' on a package with hard links..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
out2="$(install_pkg -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
r=$?
echo "${out2}" | grep -q "Force upgraded package $(basename ${PKG1} .dist) to $(basename ${PKG2} .dist) successfully" >/dev/null 2>&1
q=$?
if [ ${r} -eq 0 ] &&
   [ ${q} -eq 0 ] &&
   [ -x ${OUTPUT}/r/usr/sbin/distmk ] &&
   [ -x ${OUTPUT}/r/usr/sbin/distmk2 ] &&
   [ $(stat -c %h ${OUTPUT}/r/usr/sbin/distmk) -eq 3 ] &&
   [ $(stat -c %h ${OUTPUT}/r/usr/sbin/distmk2) -eq 3 ] &&
   [ $(stat -c %h ${OUTPUT}/r/usr/sbin/distmk3) -eq 3 ] &&
   [ "$(stat -c %i ${OUTPUT}/r/usr/sbin/distmk)" = "$(stat -c %i ${OUTPUT}/r/usr/sbin/distmk2)" ] &&
   [ "$(stat -c %i ${OUTPUT}/r/usr/sbin/distmk)" = "$(stat -c %i ${OUTPUT}/r/usr/sbin/distmk3)" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected 0 failures
[ ${FAILS} -eq 0 ] && exit 0 || exit 1
