#!/bin/sh
# Verify distadd handles multiple packages specified on the command line.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
PKG1="foo-1.0-1.$(uname -m).dist"
PKG2="bar-2.0-1.$(uname -m).dist"
PKG3="qux-3.0-1.$(uname -m).dist"

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"

# create package helper
create_pkg() {
    if [ -z "${LEAKCHECK}" ]; then
        ${DISTMK} -t ${TREE} -p "$1" -v "$2" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" >/dev/null 2>&1
    else
        ${LEAKCHECK} ${DISTMK} -t ${TREE} -p "$1" -v "$2" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}"
    fi
}

# install package helper
install_pkg() {
    [ -z "${LEAKCHECK}" ] && env ETCMERGE=${ETCMERGE} ${DISTADD} $* || env ETCMERGE=${ETCMERGE} ${LEAKCHECK} ${DISTADD} $*
}

# create the first package
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

create_pkg "foo" "1.0"

# create the second package
rm -f ${TREE}/usr/sbin/distmk
cp -p ${DISTADD} ${TREE}/usr/sbin/distadd

cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distadd root(0) root(0) 0755
EOF

create_pkg "bar" "2.0"

# create the third package
rm -f ${TREE}/usr/sbin/distadd
cp -p ${DISTINFO} ${TREE}/usr/sbin/distinfo

cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distinfo root(0) root(0) 0755
EOF

create_pkg "qux" "3.0"

#####################
# multiple packages #
#####################

# install multiple packages specified on the command line
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking 'distadd PKG1 PKG2 PKG3' works..."
out="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} ${OUTPUT}/${PKG2} ${OUTPUT}/${PKG3} 2>&1)"
r=$?
echo "${out}" | grep -q "Installed package $(basename ${PKG1} .dist) successfully" >/dev/null 2>&1
a=$?
echo "${out}" | grep -q "Installed package $(basename ${PKG2} .dist) successfully" >/dev/null 2>&1
b=$?
echo "${out}" | grep -q "Installed package $(basename ${PKG3} .dist) successfully" >/dev/null 2>&1
c=$?
if [ ${r} -eq 0 ] && [ ${a} -eq 0 ] && [ ${b} -eq 0 ] && [ ${c} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# upgrade multiple packages specified on the command line should fail
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking 'distadd -u PKG1 PKG2 PKG3' skips if packages are installed..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} ${OUTPUT}/${PKG2} ${OUTPUT}/${PKG3} 2>&1)"
out2="$(install_pkg -d -u -r ${OUTPUT}/r ${OUTPUT}/${PKG1} ${OUTPUT}/${PKG2} ${OUTPUT}/${PKG3} 2>&1)"
r=$?
echo "${out2}" | grep -q "Package foo-1.0-1 already installed, skipping." >/dev/null 2>&1
a=$?
echo "${out2}" | grep -q "Package bar-2.0-1 already installed, skipping." >/dev/null 2>&1
b=$?
echo "${out2}" | grep -q "Package qux-3.0-1 already installed, skipping." >/dev/null 2>&1
c=$?
if [ ${r} -eq 0 ] && [ ${a} -eq 0 ] && [ ${b} -eq 0 ] && [ ${c} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# upgrade force multiple packages should work
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking 'distadd -u -f PKG1 PKG2 PKG3' succeeds..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} ${OUTPUT}/${PKG2} ${OUTPUT}/${PKG3} 2>&1)"
out2="$(install_pkg -d -u -f -r ${OUTPUT}/r ${OUTPUT}/${PKG1} ${OUTPUT}/${PKG2} ${OUTPUT}/${PKG3} 2>&1)"
r=$?
echo "${out2}" | grep -q "Force upgraded package $(basename ${PKG1} .dist) to $(basename ${PKG1} .dist) successfully" >/dev/null 2>&1
a=$?
echo "${out2}" | grep -q "Force upgraded package $(basename ${PKG2} .dist) to $(basename ${PKG2} .dist) successfully" >/dev/null 2>&1
b=$?
echo "${out2}" | grep -q "Force upgraded package $(basename ${PKG3} .dist) to $(basename ${PKG3} .dist) successfully" >/dev/null 2>&1
c=$?
if [ ${r} -eq 0 ] && [ ${a} -eq 0 ] && [ ${b} -eq 0 ] && [ ${c} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected 0 failures
[ ${FAILS} -eq 0 ] && exit 0 || exit 1
