#!/bin/sh
# Verify distadd handles package scripts correctly.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
PKG1="test-1.0-1.$(uname -m).dist"
PKG2="test-2.0-1.$(uname -m).dist"

# create package helper
create_pkg() {
    if [ -z "${LEAKCHECK}" ]; then
        ${DISTMK} -t ${TREE} -p "test" -v "$1" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" >/dev/null 2>&1
    else
        ${LEAKCHECK} ${DISTMK} -t ${TREE} -p "test" -v "$1" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}"
    fi
}

# install package helper
install_pkg() {
    [ -z "${LEAKCHECK}" ] && env ETCMERGE=${ETCMERGE} ${DISTADD} $* || env ETCMERGE=${ETCMERGE} ${LEAKCHECK} ${DISTADD} $*
}

# remove package helper
remove_pkg() {
    [ -z "${LEAKCHECK}" ] && ${DISTRM} $* || ${LEAKCHECK} ${DISTRM} $*
}

# create a tree to package up
mkdir -p ${TREE}/etc
mkdir -p ${TREE}/usr/sbin
echo "System ready" > ${TREE}/etc/issue
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
etc root(0) root(0) 0755
etc/issue root(0) root(0) 0644
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"

cat << EOF > ${META}/preremove
#!/bin/sh
if [ "\${ACTION}" = "remove" ]; then
    echo "*** PREREMOVE RUNS ON REMOVE ***"
elif [ "\${ACTION}" = "upgrade" ]; then
    echo "*** PREREMOVE RUNS ON UPGRADE ***"
fi
echo "VER=\${VER}"
echo "UPGRADEVER=\${UPGRADEVER}"
EOF

chmod 0755 ${META}/preremove
create_pkg "1.0"
rm -f ${META}/preremove
create_pkg "2.0"

#############
# preremove #
#############

# preremove execution on remove
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking distadd preremove script execution on remove..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
out2="$(remove_pkg -r ${OUTPUT}/r test 2>&1)"
r=$?
msg="$(echo "${out2}" | grep '*** PREREMOVE RUNS ON REMOVE ***')"
ver="$(echo "${out2}" | grep ^VER= | cut -d '=' -f 2)"
upgradever="$(echo "${out2}" | grep ^UPGRADEVER= | cut -d '=' -f 2)"
if [ ${r} -eq 0 ] && [ "${msg}" = "*** PREREMOVE RUNS ON REMOVE ***" ] && [ "${ver}" = "1.0" ] && [ -z "${upgradever}" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# preremove execution on upgrade
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking distadd preremove script execution on upgrade..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
out2="$(install_pkg -d -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
r=$?
msg="$(echo "${out2}" | grep '*** PREREMOVE RUNS ON UPGRADE ***')"
if [ ${r} -eq 0 ] && [ "${msg}" = "*** PREREMOVE RUNS ON UPGRADE ***" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# preremove execution on upgrade, VER != UPGRADEVER
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking distadd preremove script execution on upgrade (VER != UPGRADEVER)..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
out2="$(install_pkg -d -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
r=$?
msg="$(echo "${out2}" | grep '*** PREREMOVE RUNS ON UPGRADE ***')"
ver="$(echo "${out2}" | grep ^VER= | cut -d '=' -f 2)"
upgradever="$(echo "${out2}" | grep ^UPGRADEVER= | cut -d '=' -f 2)"
if [ ${r} -eq 0 ] && [ "${msg}" = "*** PREREMOVE RUNS ON UPGRADE ***" ] && [ "${ver}" = "1.0" ] && [ "${upgradever}" = "2.0" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected 0 failures
[ ${FAILS} -eq 0 ] && exit 0 || exit 1
