#!/bin/sh
# Verify distadd handles package scripts correctly.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
PKG1="test-1.0-1.$(uname -m).dist"
PKG2="test-2.0-1.$(uname -m).dist"
PKG3="test-3.0-1.$(uname -m).dist"
PKG4="test-4.0-1.$(uname -m).dist"
PKG5="test-5.0-1.$(uname -m).dist"
PKG6="test-6.0-1.$(uname -m).dist"

# create package helper
create_pkg() {
    if [ -z "${LEAKCHECK}" ]; then
        ${DISTMK} -t ${TREE} -p "test" -v "$1" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" >/dev/null 2>&1
    else
        ${LEAKCHECK} ${DISTMK} -t ${TREE} -p "test" -v "$1" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}"
    fi
}

# install package helper
install_pkg() {
    [ -z "${LEAKCHECK}" ] && env ETCMERGE=${ETCMERGE} ${DISTADD} $* || env ETCMERGE=${ETCMERGE} ${LEAKCHECK} ${DISTADD} $*
}

# create a tree to package up
mkdir -p ${TREE}/etc
mkdir -p ${TREE}/usr/sbin
echo "System ready" > ${TREE}/etc/issue
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
etc root(0) root(0) 0755
etc/issue root(0) root(0) 0644
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"

cat << EOF > ${META}/preinstall
#!/bin/sh
if [ "\${ACTION}" = "install" ]; then
    echo "*** PREINSTALL RUNS ON INSTALL ***"
elif [ "\${ACTION}" = "upgrade" ]; then
    echo "*** PREINSTALL RUNS ON UPGRADE ***"
fi
EOF

cat << EOF > ${META}/postinstall
#!/bin/sh
if [ "\${ACTION}" = "install" ]; then
    echo "*** POSTINSTALL RUNS ON INSTALL ***"
elif [ "\${ACTION}" = "upgrade" ]; then
    echo "*** POSTINSTALL RUNS ON UPGRADE ***"
fi
EOF

cat << EOF > ${META}/preremove
#!/bin/sh
if [ "\${ACTION}" = "remove" ]; then
    echo "*** PREREMOVE RUNS ON REMOVE ***"
elif [ "\${ACTION}" = "upgrade" ]; then
    echo "*** PREREMOVE RUNS ON UPGRADE ***"
fi
EOF

chmod 0755 ${META}/preinstall
chmod 0755 ${META}/postinstall
chmod 0755 ${META}/preremove
create_pkg "1.0"
create_pkg "2.0"
rm -f ${META}/preinstall
rm -f ${META}/postinstall
rm -f ${META}/preremove

cat << EOF > ${META}/preinstall
#!/bin/sh
exit 1
EOF

chmod 0755 ${META}/preinstall
create_pkg "3.0"
rm -f ${META}/preinstall

cat << EOF > ${META}/postinstall
#!/bin/sh
exit 1
EOF

chmod 0755 ${META}/postinstall
create_pkg "4.0"
rm -f ${META}/postinstall

cat << EOF > ${META}/preremove
#!/bin/sh
exit 1
EOF

chmod 0755 ${META}/preremove
create_pkg "5.0"
create_pkg "6.0"
rm -f ${META}/preremove

##############
# preinstall #
##############

# preinstall execution on install
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking distadd preinstall script execution on install..."
out="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
r=$?
echo "${out}" | grep -q "\*\*\* PREINSTALL RUNS ON INSTALL \*\*\*" >/dev/null 2>&1
a=$?
if [ ${r} -eq 0 ] && [ ${a} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# preinstall execution on upgrade
echo -n "checking distadd preinstall script execution on upgrade..."
out="$(install_pkg -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
r=$?
echo "${out}" | grep -q "\*\*\* PREINSTALL RUNS ON UPGRADE \*\*\*" >/dev/null 2>&1
a=$?
if [ ${r} -eq 0 ] && [ ${a} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# preinstall skip on install
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking distadd preinstall script skip on install..."
out="$(install_pkg -d -S -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
r=$?
echo "${out}" | grep -q "\*\*\* PREINSTALL RUNS ON INSTALL \*\*\*" >/dev/null 2>&1
a=$?
if [ ${r} -eq 0 ] && [ ${a} -eq 1 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# preinstall skip on upgrade
echo -n "checking distadd preinstall script skip on upgrade..."
out="$(install_pkg -d -f -S -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
r=$?
echo "${out}" | grep -q "\*\*\* PREINSTALL RUNS ON UPGRADE \*\*\*" >/dev/null 2>&1
a=$?
if [ ${r} -eq 0 ] && [ ${a} -eq 1 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# preinstall fails, but continue install
rm -rf ${OUTPUT}/r2
mkdir -p ${OUTPUT}/r2

echo -n "checking 'distadd -F' on preinstall script failure..."
out="$(install_pkg -d -F -r ${OUTPUT}/r2 ${OUTPUT}/${PKG3} 2>&1)"
r=$?
echo "${out}" | grep -q "preinstall failed for test, continuing anyway" >/dev/null 2>&1
a=$?
if [ ${r} -eq 0 ] && [ ${a} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

###############
# postinstall #
###############

# postinstall execution on install
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking distadd postinstall script execution on install..."
out="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
r=$?
echo "${out}" | grep -q "\*\*\* POSTINSTALL RUNS ON INSTALL \*\*\*" >/dev/null 2>&1
a=$?
if [ ${r} -eq 0 ] && [ ${a} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# postinstall execution on upgrade
echo -n "checking distadd postinstall script execution on upgrade..."
out="$(install_pkg -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
r=$?
echo "${out}" | grep -q "\*\*\* POSTINSTALL RUNS ON UPGRADE \*\*\*" >/dev/null 2>&1
a=$?
if [ ${r} -eq 0 ] && [ ${a} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# postinstall skip on install
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking distadd postinstall script skip on install..."
out="$(install_pkg -d -S -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
r=$?
echo "${out}" | grep -q "\*\*\* POSTINSTALL RUNS ON INSTALL \*\*\*" >/dev/null 2>&1
a=$?
if [ ${r} -eq 0 ] && [ ${a} -eq 1 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# postinstall skip on upgrade
echo -n "checking distadd postinstall script skip on upgrade..."
out="$(install_pkg -d -f -S -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
r=$?
echo "${out}" | grep -q "\*\*\* POSTINSTALL RUNS ON UPGRADE \*\*\*" >/dev/null 2>&1
a=$?
if [ ${r} -eq 0 ] && [ ${a} -eq 1 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# postinstall fails, but continue install
rm -rf ${OUTPUT}/r2
mkdir -p ${OUTPUT}/r2

echo -n "checking 'distadd -F' on postinstall script failure..."
out="$(install_pkg -d -F -r ${OUTPUT}/r2 ${OUTPUT}/${PKG4} 2>&1)"
r=$?
echo "${out}" | grep -q "postinstall failed for test, continuing anyway" >/dev/null 2>&1
a=$?
if [ ${r} -eq 0 ] && [ ${a} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

#############
# preremove #
#############

# preremove execution on upgrade
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking distadd preremove script execution on upgrade..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
out2="$(install_pkg -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
r=$?
echo "${out2}" | grep -q "\*\*\* PREREMOVE RUNS ON UPGRADE \*\*\*" >/dev/null 2>&1
if [ ${r} -eq 0 ] && [ ${a} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# preremove skip on upgrade
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking distadd preremove script skip on upgrade..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
out2="$(install_pkg -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
r=$?
echo "${out2}" | grep -q "\*\*\* PREREMOVE RUNS ON UPGRADE \*\*\*" >/dev/null 2>&1
if [ ${r} -eq 0 ] && [ ${a} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# preremove fails, but continue install
rm -rf ${OUTPUT}/r2
mkdir -p ${OUTPUT}/r2

echo -n "checking 'distadd -F' on preremove script failure..."
out1="$(install_pkg -d -r ${OUTPUT}/r2 ${OUTPUT}/${PKG5} 2>&1)"
out2="$(install_pkg -d -f -F -u -r ${OUTPUT}/r2 ${OUTPUT}/${PKG6} 2>&1)"
r=$?
echo "${out2}" | grep -q "preremove failed for test, continuing anyway" >/dev/null 2>&1
a=$?
if [ ${r} -eq 0 ] && [ ${a} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected 0 failures
[ ${FAILS} -eq 0 ] && exit 0 || exit 1
