#!/bin/sh
# Verify distadd can handle symlinks.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
PKG1="test-1.0-1.$(uname -m).dist"
PKG2="test-2.0-1.$(uname -m).dist"
PKG3="test-3.0-1.$(uname -m).dist"

# create package helper
create_pkg() {
    if [ -z "${LEAKCHECK}" ]; then
        ${DISTMK} -t ${TREE} -p "test" -v "$1" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" >/dev/null 2>&1
    else
        ${LEAKCHECK} ${DISTMK} -t ${TREE} -p "test" -v "$1" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}"
    fi
}

# install package helper
install_pkg() {
    [ -z "${LEAKCHECK}" ] && env ETCMERGE=${ETCMERGE} ${DISTADD} $* || env ETCMERGE=${ETCMERGE} ${LEAKCHECK} ${DISTADD} $*
}

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk
( cd ${TREE}/usr/sbin ; ln -s distmk distmk2 )

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
usr/sbin/distmk2 root(0) root(0) symlink(distmk)
EOF

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"

create_pkg "1.0"

# make a variant package with different file symlinks
( cd ${TREE}/usr/sbin ; ln -sf distmk distmk3 )
( cd ${TREE}/usr/sbin ; ln -sf distmk3 distmk2 )
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
usr/sbin/distmk2 root(0) root(0) symlink(distmk3)
usr/sbin/distmk3 root(0) root(0) symlink(distmk)
EOF

create_pkg "2.0"

# make a variant package with directory symlinks
rm -f ${TREE}/usr/sbin/distmk2
rm -f ${TREE}/usr/sbin/distmk3
( cd ${TREE}/usr ; ln -sf sbin sbin2 )
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
usr/sbin2 root(0) root(0) symlink(sbin)
EOF

create_pkg "3.0"

# install package with symlinks
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking 'distadd' on package with file symlinks..."
out="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
r=$?
echo "${out}" | grep -q "Installed package $(basename ${PKG1} .dist) successfully" >/dev/null 2>&1
a=$?
if [ ${r} -eq 0 ] &&
   [ ${a} -eq 0 ] &&
   [ -x ${OUTPUT}/r/usr/sbin/distmk ] &&
   [ -L ${OUTPUT}/r/usr/sbin/distmk2 ] &&
   [ "$(readlink ${OUTPUT}/r/usr/sbin/distmk2)" = "distmk" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# upgrade a package with symlinks to a package with symlinks
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking 'distadd -u' on a package with file symlinks..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
out2="$(install_pkg -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
r=$?
echo "${out2}" | grep -q "Force upgraded package $(basename ${PKG1} .dist) to $(basename ${PKG2} .dist) successfully" >/dev/null 2>&1
a=$?
if [ ${r} -eq 0 ] &&
   [ ${a} -eq 0 ] &&
   [ -x ${OUTPUT}/r/usr/sbin/distmk ] &&
   [ -L ${OUTPUT}/r/usr/sbin/distmk2 ] &&
   [ -L ${OUTPUT}/r/usr/sbin/distmk3 ] &&
   [ "$(readlink ${OUTPUT}/r/usr/sbin/distmk2)" = "distmk3" ] &&
   [ "$(readlink ${OUTPUT}/r/usr/sbin/distmk3)" = "distmk" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# upgrade a package with symlinks to a package with directory symlinks
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r

echo -n "checking 'distadd -u' on a package with directory symlinks..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
out2="$(install_pkg -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG3} 2>&1)"
r=$?
echo "${out2}" | grep -q "Force upgraded package $(basename ${PKG1} .dist) to $(basename ${PKG3} .dist) successfully" >/dev/null 2>&1
a=$?
if [ ${r} -eq 0 ] &&
   [ ${a} -eq 0 ] &&
   [ -x ${OUTPUT}/r/usr/sbin/distmk ] &&
   [ ! -r ${OUTPUT}/r/usr/sbin/distmk2 ] &&
   [ -L ${OUTPUT}/r/usr/sbin2 ] &&
   [ "$(readlink ${OUTPUT}/r/usr/sbin2)" = "sbin" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected 0 failures
[ ${FAILS} -eq 0 ] && exit 0 || exit 1
