#!/bin/sh
# Verify distadd can upgrade packages.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
PKG1="test-1.0-1.$(uname -m).dist"
PKG2="test-2.0-1.$(uname -m).dist"
PKG3="test-3.0-1.$(uname -m).dist"

# create package helper
create_pkg() {
    if [ -z "${LEAKCHECK}" ]; then
        ${DISTMK} -t ${TREE} -p "test" -v "$1" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" >/dev/null 2>&1
    else
        ${LEAKCHECK} ${DISTMK} -t ${TREE} -p "test" -v "$1" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}"
    fi
}

# install package helper
install_pkg() {
    [ -z "${LEAKCHECK}" ] && env ETCMERGE=${ETCMERGE} ${DISTADD} $* || env ETCMERGE=${ETCMERGE} ${LEAKCHECK} ${DISTADD} $*
}

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"

create_pkg "1.0"

# make a variant package to upgrade to
mv ${TREE}/usr/sbin/distmk ${TREE}/usr/sbin/distmk2
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk2 root(0) root(0) 0755
EOF

create_pkg "2.0"

# make a variant package to upgrade to (same filename)
echo "different file" > ${TREE}/usr/sbin/distmk2

create_pkg "3.0"

# upgrade same package to same package (error)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distadd -u' on same NVR..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
out2="$(install_pkg -d -u -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
r=$?
echo "${out2}" | grep -q "Package test-1.0-1 already installed, skipping." >/dev/null 2>&1
a=$?
if [ ${r} -eq 0 ] && [ ${a} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# upgrade same package to same package (force)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distadd -u -f' on same NVR..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
out2="$(install_pkg -d -u -f -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
r=$?
echo "${out2}" | grep -q "Force upgraded package $(basename ${PKG1} .dist) to $(basename ${PKG1} .dist) successfully" >/dev/null
a=$?
p1=$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM packages" | wc -l)
if [ ${r} -eq 0 ] && [ ${a} -eq 0 ] && [ -f ${OUTPUT}/r/usr/sbin/distmk ] && [ ${p1} -eq 1 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# upgrade to new package
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distadd -u' to new NVR..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
out2="$(${DISTADD} -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
r=$?
echo "${out2}" | grep -q "Force upgraded package $(basename ${PKG1} .dist) to $(basename ${PKG2} .dist) successfully" >/dev/null 2>&1
a=$?
p1=$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM PACKAGES" | wc -l)
p2="$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM PACKAGES" | grep "1|test|2.0|1")"
if [ ${r} -eq 0 ] && [ ${a} -eq 0 ] && [ -f ${OUTPUT}/r/usr/sbin/distmk2 ] && [ ! -f ${OUTPUT}/r/usr/sbin/distmk ] && [ ${p1} -eq 1 ] && [ ! -z "${p2}" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# upgrade to new package (same filename)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distadd -u' to new NVR (same filename)..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
out2="$(${DISTADD} -d -f -u -r ${OUTPUT}/r ${OUTPUT}/${PKG3} 2>&1)"
r=$?
echo "${out2}" | grep -q "Force upgraded package $(basename ${PKG2} .dist) to $(basename ${PKG3} .dist) successfully" >/dev/null 2>&1
a=$?
p1=$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM PACKAGES" | wc -l)
p2="$(sqlite ${OUTPUT}/r/var/db/dist/installed.db "SELECT * FROM PACKAGES" | grep "1|test|3.0|1")"
if [ ${r} -eq 0 ] && [ ${a} -eq 0 ] && [ -f ${OUTPUT}/r/usr/sbin/distmk2 ] && [ ${p1} -eq 1 ] && [ ! -z "${p2}" ] && [ "$(cat ${OUTPUT}/r/usr/sbin/distmk2)" = "different file" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected 0 failures
[ ${FAILS} -eq 0 ] && exit 0 || exit 1
