#!/bin/sh
# Verify we can use wildcards with the distadd command.

CWD="$(pwd)"

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
EXPECTED_FAILS=0
PKG1="foo-1.0-1.$(uname -m).dist"
PKG2="bar-2.0-1.$(uname -m).dist"
PKG3="baz-3.0-1.$(uname -m).dist"
PKG4="qux-4.0-1.$(uname -m).dist"

# create package helper
create_pkg() {
    if [ -z "${LEAKCHECK}" ]; then
        ${DISTMK} -t ${TREE} -p "$1" -v "$2" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" >/dev/null 2>&1
    else
        ${LEAKCHECK} ${DISTMK} -t ${TREE} -p "$1" -v "$2" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}"
    fi
}

# install package helper
install_pkg() {
    [ -z "${LEAKCHECK}" ] && env ETCMERGE=${ETCMERGE} ${DISTADD} $* || env ETCMERGE=${ETCMERGE} ${LEAKCHECK} ${DISTADD} $*
}

# meta files
DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"

# PKG1:
rm -rf ${TREE}/*
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

create_pkg "foo" "1.0"

# PKG2:
rm -rf ${TREE}/*
mkdir -p ${TREE}/usr/bin
cp -p ${DISTMK} ${TREE}/usr/bin/distmk

cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/bin root(0) root(0) 0755
usr/bin/distmk root(0) root(0) 0755
EOF

create_pkg "bar" "2.0"

# PKG3:
rm -rf ${TREE}/*
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTADD} ${TREE}/usr/sbin/distadd

cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distadd root(0) root(0) 0755
EOF

create_pkg "baz" "3.0"

# PKG4:
rm -rf ${TREE}/*
mkdir -p ${TREE}/usr/bin
cp -p ${DISTADD} ${TREE}/usr/bin/distadd

cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/bin root(0) root(0) 0755
usr/bin/distadd root(0) root(0) 0755
EOF

create_pkg "qux" "4.0"

# install multiple packages by * wildcard
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distadd *.dist' on package files..."
out="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/*.dist 2>&1)"
r=$?
count="$(echo "${out}" | grep "^Install" | wc -l)"
echo "${out}" | grep -q "Installed package foo-1.0-1" >/dev/null 2>&1
f1=$?
echo "${out}" | grep -q "Installed package bar-2.0-1" >/dev/null 2>&1
f2=$?
echo "${out}" | grep -q "Installed package baz-3.0-1" >/dev/null 2>&1
f3=$?
echo "${out}" | grep -q "Installed package qux-4.0-1" >/dev/null 2>&1
f4=$?
if [ ${r} -eq 0 ] && [ ${count} -eq 8 ] && [ ${f1} -eq 0 ] && [ ${f2} -eq 0 ] && [ ${f3} -eq 0 ] && [ ${f4} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# install multiple packages with {} brace expansion
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distadd {}' expansion on package files..."
out="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/{foo,qux}*dist 2>&1)"
r=$?
count="$(echo "${out}" | grep "^Install" | wc -l)"
echo "${out}" | grep -q "Installed package foo-1.0-1" >/dev/null 2>&1
f1=$?
echo "${out}" | grep -q "Installed package bar-2.0-1" >/dev/null 2>&1
f2=$?
echo "${out}" | grep -q "Installed package baz-3.0-1" >/dev/null 2>&1
f3=$?
echo "${out}" | grep -q "Installed package qux-4.0-1" >/dev/null 2>&1
f4=$?
if [ ${r} -eq 0 ] && [ ${count} -eq 4 ] && [ ${f1} -eq 0 ] && [ ${f2} -eq 1 ] && [ ${f3} -eq 1 ] && [ ${f4} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected $EXPECTED_FAILS failures
[ ${FAILS} -eq ${EXPECTED_FAILS} ] && exit 0 || exit 1
