#!/bin/sh
# Verify distchk works on packages with skipped checksums for files.

CWD="$(pwd)"

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
PKG="test-1.0-1.$(uname -m).dist"

# create package helper
create_pkg() {
    if [ -z "${LEAKCHECK}" ]; then
        ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" >/dev/null 2>&1
    else
        ${LEAKCHECK} ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}"
    fi
}

# install package helper
install_pkg() {
    [ -z "${LEAKCHECK}" ] && env ETCMERGE=${ETCMERGE} ${DISTADD} $* || env ETCMERGE=${ETCMERGE} ${LEAKCHECK} ${DISTADD} $*
}

# check package hlper
check_pkg() {
    [ -z "${LEAKCHECK}" ] && ${DISTCHK} $* || ${LEAKCHECK} ${DISTCHK} $*
}

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk
cp -p ${DISTMK} ${TREE}/usr/sbin/mkdist

# create plist
USER="$(id -un)"
tUID="$(id -ur)"
GROUP="$(id -gn)"
tGID="$(id -gr)"

echo "usr ${USER}(${tUID}) ${GROUP}(${tGID}) 0755" > ${META}/plist
echo "usr/sbin ${USER}(${tUID}) ${GROUP}(${tGID}) 0755" >> ${META}/plist
echo "usr/sbin/distmk ${USER}(${tUID}) ${GROUP}(${tGID}) 0755 nochecksum" >> ${META}/plist
echo "usr/sbin/mkdist ${USER}(${tUID}) ${GROUP}(${tGID}) 0755" >> ${META}/plist

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"

create_pkg

# check installed package with nochecksum used
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distchk PKG' on a nochecksum package..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(check_pkg -r ${OUTPUT}/r test 2>&1)"
r=$?
echo "${out2}" | grep -q "^OK$" >/dev/null 2>&1
a=$?
if [ ${r} -eq 0 ] && [ ${a} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

[ ${FAILS} -gt 0 ] && exit 1 || exit 0
