#!/bin/sh -x
# Verify we can get information about installed packages

CWD="$(pwd)"

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
EXPECTED_FAILS=0
PKG="test-1.0-1.$(uname -m).dist"

# create package helper
create_pkg() {
    if [ -z "${LEAKCHECK}" ]; then
        ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" >/dev/null 2>&1
    else
        ${LEAKCHECK} ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}"
    fi
}

# install package helper
install_pkg() {
    [ -z "${LEAKCHECK}" ] && env ETCMERGE=${ETCMERGE} ${DISTADD} $* || env ETCMERGE=${ETCMERGE} ${LEAKCHECK} ${DISTADD} $*
}

# info package helper
info_pkg() {
    [ -z "${LEAKCHECK}" ] && ${DISTINFO} $* || ${LEAKCHECK} ${DISTINFO} $*
}

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"

# create the package
create_pkg

# list info about a package file
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distinfo PKG' on a package file..."
out="$(info_pkg ${OUTPUT}/${PKG} 2>&1)"
r=$?
f1="$(echo "${out}" | grep ^peachtree | awk '{ print $1; }')"
f2="$(echo "${out}" | grep ^peachtree | awk '{ print $2; }')"
if [ ${r} -eq 0 ] && [ "${f1}" = "peachtree" ] && [ "${f2}" = "test-1.0-1.$(uname -m)" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# list all packages
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distinfo' to list all installed packages..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(info_pkg -r ${OUTPUT}/r 2>&1)"
r=$?
f1="$(echo "${out2}" | grep ^peachtree | awk '{ print $1; }')"
f2="$(echo "${out2}" | grep ^peachtree | awk '{ print $2; }')"
if [ ${r} -eq 0 ] && [ "${f1}" = "peachtree" ] && [ "${f2}" = "test-1.0-1.$(uname -m)" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# list a non-existent package
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distinfo PKG' for a non-existent PKG..."
out1="$(info_pkg -r ${OUTPUT}/r test2 2>&1)"
r=$?
out2="$(echo "${out1}" | grep ^peachtree)"
if [ ${r} -eq 0 ] && [ -z "${out2}" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out1}"

# list package version
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distinfo -v'..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(info_pkg -v -r ${OUTPUT}/r test 2>&1)"
r=$?
r1="$(echo "${out2}" | grep "Name:" | cut -c17-)"
r2="$(echo "${out2}" | grep "Version:" | cut -c17-)"
r3="$(echo "${out2}" | grep "Release:" | cut -c17-)"
if [ ${r} -eq 0 ] && [ "${r1}" = "test" ] && [ "${r2}" = "1.0" ] && [ "${r3}" = "1" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# list package maintainer
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distinfo -m'..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(info_pkg -m -r ${OUTPUT}/r test 2>&1)"
r=$?
r1="$(echo "${out2}" | grep "Name:" | cut -c17-)"
r2="$(echo "${out2}" | grep "Maintainer:" | cut -c17-)"
if [ ${r} -eq 0 ] && [ "${r1}" = "test" ] && [ "${r2}" = "J. Random Packager <someone@example.net>" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# list package description
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distinfo -d'..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(info_pkg -d -r ${OUTPUT}/r test 2>&1)"
r=$?
r1="$(echo "${out2}" | grep "Name:" | cut -c17-)"
r2="$(echo "${out2}" | grep "^Test program$")"
if [ ${r} -eq 0 ] && [ "${r1}" = "test" ] && [ "${r2}" = "Test program" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# list package size
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distinfo -s'..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(info_pkg -s -r ${OUTPUT}/r test 2>&1)"
r=$?
r1="$(echo "${out2}" | grep "Name:" | cut -c17-)"
r2="$(echo "${out2}" | grep "Installed size:" | grep "bytes")"
if [ ${r} -eq 0 ] && [ "${r1}" = "test" ] && [ ! -z "${r2}" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# list package pkgsize (on installed package)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distinfo -p' on an installed package..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(info_pkg -p -r ${OUTPUT}/r test 2>&1)"
r=$?
r1="$(echo "${out2}" | grep "Name:" | cut -c17-)"
if [ ${r} -eq 0 ] && [ "${r1}" = "test" ] && [ ! -z "${r2}" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# list package pkgsize (on a package file)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distinfo -p PKG' on a package file..."
out1="$(info_pkg -p ${OUTPUT}/${PKG} 2>&1)"
r=$?
r1="$(echo "${out1}" | grep "Package:" | cut -c17-)"
r2="$(echo "${out1}" | grep "Name:" | cut -c17-)"
r3="$(echo "${out1}" | grep "Package size:" | grep "bytes")"
if [ ${r} -eq 0 ] && [ "$(basename ${r1})" = "${PKG}" ] && [ "${r2}" = "test" ] && [ ! -z "${r3}" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out1}"

# list package arch
echo -n "checking 'distinfo -a'..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(info_pkg -a -r ${OUTPUT}/r test 2>&1)"
r=$?
r1="$(echo "${out2}" | grep "Name:" | cut -c17-)"
r2="$(echo "${out2}" | grep "Architecture:" | cut -c17-)"
if [ ${r} -eq 0 ] && [ "${r1}" = "test" ] && [ "${r2}" = "$(uname -m)" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# list package files
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distinfo -f'..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(info_pkg -f -r ${OUTPUT}/r test 2>&1)"
r=$?
r1="$(echo "${out2}" | grep "Name:" | cut -c17-)"
r2="$(echo "${out2}" | grep -E "^/usr$")"
r3="$(echo "${out2}" | grep -E "^/usr/sbin$")"
r4="$(echo "${out2}" | grep -E "^/usr/sbin/distmk$")"
if [ ${r} -eq 0 ] && [ "${r1}" = "test" ] && [ "${r2}" = "/usr" ] && [ "${r3}" = "/usr/sbin" ] && [ "${r4}" = "/usr/sbin/distmk" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# list package license
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distinfo -l'..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(info_pkg -l -r ${OUTPUT}/r test 2>&1)"
r=$?
r1="$(echo "${out2}" | grep "Name:" | cut -c17-)"
r2="$(echo "${out2}" | grep "License:" | cut -c17-)"
if [ ${r} -eq 0 ] && [ "${r1}" = "test" ] && [ "${r2}" = "WTFPL" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# list package repository name
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distinfo -R'..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(info_pkg -R -r ${OUTPUT}/r test 2>&1)"
r=$?
r1="$(echo "${out2}" | grep "Name:" | cut -c17-)"
r2="$(echo "${out2}" | grep "Repository:" | cut -c17-)"
if [ ${r} -eq 0 ] && [ "${r1}" = "test" ] && [ "${r2}" = "peachtree" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# is this a debug package or not?
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distinfo -g'..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(info_pkg -g -r ${OUTPUT}/r test 2>&1)"
r=$?
r1="$(echo "${out2}" | grep "Name:" | cut -c17-)"
r2="$(echo "${out2}" | grep "Debug package:" | cut -c17-)"
if [ ${r} -eq 0 ] && [ "${r1}" = "test" ] && [ "${r2}" = "no" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# list all package info
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distinfo -A'..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(info_pkg -A -r ${OUTPUT}/r test 2>&1)"
r=$?
r1="$(echo "${out2}" | grep "Name:" | cut -c17-)"
r2="$(echo "${out2}" | grep "Version:" | cut -c17-)"
r3="$(echo "${out2}" | grep "Release:" | cut -c17-)"
r4="$(echo "${out2}" | grep "License:" | cut -c17-)"
r5="$(echo "${out2}" | grep "Architecture:" | cut -c17-)"
r6="$(echo "${out2}" | grep "Maintainer:" | cut -c17-)"
r7="$(echo "${out2}" | grep "Installed size:" | grep "bytes")"
r8="$(echo "${out2}" | grep "^Test program$")"
if [ ${r} -eq 0 ] &&
   [ "${r1}" = "test" ] &&
   [ "${r2}" = "1.0" ] &&
   [ "${r3}" = "1" ] &&
   [ "${r4}" = "WTFPL" ] &&
   [ "${r5}" = "$(uname -m)" ] &&
   [ "${r6}" = "J. Random Packager <someone@example.net>" ] &&
   [ ! -z "${r7}" ] &&
   [ "${r8}" = "Test program" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# query for package that owns a file (expect success)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distinfo -q FILE' with expected success..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(info_pkg -r ${OUTPUT}/r -q /usr/sbin/distmk 2>&1)"
r=$?
r1="$(echo "${out2}" | grep "/usr/sbin/distmk: " | awk '{ print $2; }')"
if [ ${r} -eq 0 ] && [ "${r1}" = "$(basename ${PKG} .dist)" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# query for package that owns a file (expect failure)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distinfo -q FILE' with expected failure..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(info_pkg -r ${OUTPUT}/r -q /usr/sbin/thing 2>&1)"
r=$?
echo "${out2}" | grep -q "/usr/sbin/thing: No such file or directory" >/dev/null 2>&1
f1=$?
if [ ${r} -eq 0 ] && [ ${f1} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# list info in shell variable output (make sure this is the last test)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distinfo -A -S' for shell output..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
info_pkg -A -S -r ${OUTPUT}/r test 2>&1 | grep -v "^==" > ${OUTPUT}/infovars
r=$?
. ${OUTPUT}/infovars
if [ ${r} -eq 0 ] &&
   [ "${PKG}" = "test" ] &&
   [ "${VER}" = "1.0" ] &&
   [ "${REL}" = "1" ] &&
   [ "${LICENSE}" = "WTFPL" ] &&
   [ "${REPO}" = "peachtree" ] &&
   [ "${ARCH}" = "$(uname -m)" ] &&
   [ "${MAINTAINER}" = "J. Random Packager <someone@example.net>" ] &&
   [ "${DESC}" = "Test program" ] &&
   [ ${SIZE} -gt 0 ] &&
   [ "${DEBUG}" = "n" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    cat ${OUTPUT}/infovars
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected $EXPECTED_FAILS failures
[ ${FAILS} -eq ${EXPECTED_FAILS} ] && exit 0 || exit 1
