#!/bin/sh
# Verify the --provs option works on distinfo.

CWD="$(pwd)"

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
EXPECTED_FAILS=0

# create a tree to package up
mkdir -p ${TREE}/usr/lib
cp /usr/lib/libc.so.6 ${TREE}/usr/lib/libc.so.6
chmod 0755 ${TREE}/usr/lib/libc.so.6

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/lib root(0) root(0) 0755
usr/lib/libc.so.6 root(0) root(0) 0755
EOF

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"
PKG="test-1.0-1.$(uname -m).dist"

# create package helper
create_pkg() {
    if [ -z "${LEAKCHECK}" ]; then
        ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" >/dev/null 2>&1
    else
        ${LEAKCHECK} ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}"
    fi
}

# info package helper
info_pkg() {
    [ -z "${LEAKCHECK}" ] && ${DISTINFO} $* || ${LEAKCHECK} ${DISTINFO} $*
}

# create the package
create_pkg

# list the ELF shared libraries provided by this package
echo -n "checking 'distinfo --provs PKG' on a package file..."
out="$(info_pkg --provs ${OUTPUT}/${PKG} 2>&1)"
r=$?
echo "${out}" | grep -q "^libc.so.6" >/dev/null 2>&1
q=$?
if [ ${r} -eq 0 ] && [ ${q} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# ensure --provs only works for package files
echo -n "checking 'distinfo --provs PKG' on an installed package..."
out="$(info_pkg --provs glibc 2>&1)"
r=$?
echo "${out}" | grep -q "The --provs and --deps options are only valid on package files." >/dev/null 2>&1
q=$?
if [ ${r} -eq 1 ] && [ ${q} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected $EXPECTED_FAILS failures
[ ${FAILS} -eq ${EXPECTED_FAILS} ] && exit 0 || exit 1
