#!/bin/sh
# Verify we can use wildcards with the distinfo command.

CWD="$(pwd)"

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
EXPECTED_FAILS=0
PKG1="test-1.0-1.$(uname -m).dist"
PKG2="test-2.0-1.$(uname -m).dist"
PKG3="test-3.0-1.$(uname -m).dist"

# create package helper
create_pkg() {
    if [ -z "${LEAKCHECK}" ]; then
        ${DISTMK} -t ${TREE} -p "test" -v "$1" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" >/dev/null 2>&1
    else
        ${LEAKCHECK} ${DISTMK} -t ${TREE} -p "test" -v "$1" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}"
    fi
}

# info package helper
info_pkg() {
    [ -z "${LEAKCHECK}" ] && ${DISTINFO} $* || ${LEAKCHECK} ${DISTINFO} $*
}

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"

# create the packages
create_pkg "1.0"
create_pkg "2.0"
create_pkg "3.0"

# list info about package files
echo -n "checking 'distinfo *.dist' on package files..."
out="$(info_pkg ${OUTPUT}/*.dist 2>&1)"
r=$?
count="$(echo "${out}" | grep -v "^==" | wc -l)"
echo "${out}" | grep -q "test-1.0" >/dev/null 2>&1
f1=$?
echo "${out}" | grep -q "test-2.0" >/dev/null 2>&1
f2=$?
echo "${out}" | grep -q "test-3.0" >/dev/null 2>&1
f3=$?
if [ ${r} -eq 0 ] && [ ${count} -eq 3 ] && [ ${f1} -eq 0 ] && [ ${f2} -eq 0 ] && [ ${f3} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected $EXPECTED_FAILS failures
[ ${FAILS} -eq ${EXPECTED_FAILS} ] && exit 0 || exit 1
