#!/bin/sh
# Make sure all checksum types are honored

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"
PKG="test-1.0-1.$(uname -m).dist"

CWD=$(pwd)

# create the package
create_pkg() {
    cmd="${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" -s $1"
    [ -z "${LEAKCHECK}" ] && ${cmd} >/dev/null 2>&1 || ${LEAKCHECK} ${cmd}
}

# check each digest type
for t in md5 sha1 sha256 sha384 sha512 ; do
    echo -n "checking ${t} support..."
    create_pkg ${t}
    cd ${OUTPUT}
    tar -xf ${PKG}
    sum="$(sqlite bom.db "SELECT digest FROM contents WHERE DIGEST NOT NULL")"
    computed="$(${t}sum ${TREE}/usr/sbin/distmk | cut -d ' ' -f 1)"

    if [ "${sum}" = "${computed}" ]; then
        echo "passed."
    else
        FAILS=$((${FAILS} + 1))
        echo "FAILED."
    fi

    cd ${CWD}
    rm -rf ${OUTPUT}/*
done

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected zero failures
[ ${FAILS} -gt 0 ] && exit 1 || exit 0
