#!/bin/sh
# Make sure all compression types are honored

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"
PKG="test-1.0-1.$(uname -m).dist"

CWD=$(pwd)

# create the package
create_pkg() {
    cmd="${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" -z $1"
    [ -z "${LEAKCHECK}" ] && ${cmd} >/dev/null 2>&1 || ${LEAKCHECK} ${cmd}
}

# check each compression type
for t in bzip2 bz2 gzip gz zstd none ; do
    echo -n "checking ${t} support..."
    cd ${OUTPUT}
    create_pkg ${t}
    tar -xf ${PKG}
    r=0

    case "${t}" in
        bzip2|bz2)
            # bzip2 does not have a "-l" option, so use "-t"
            bzip2 -t ${OUTPUT}/reloc.payload >/dev/null 2>&1 || r=1 ;;
        gzip|gz)
            gzip -l ${OUTPUT}/reloc.payload >/dev/null 2>&1 || r=1 ;;
        zstd)
            zstd -l ${OUTPUT}/reloc.payload >/dev/null 2>&1 || r=1 ;;
        none)
            tar -tvf ${OUTPUT}/reloc.payload >/dev/null 2>&1 || r=1 ;;
    esac

    FAILS=$((${FAILS} + ${r}))
    [ $r -eq 0 ] && echo "passed." || echo "FAILED."
    cd ${CWD}
    rm -rf ${OUTPUT}/*
done

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected zero failures
[ ${FAILS} -gt 0 ] && exit 1 || exit 0
