#!/bin/sh
# Make sure 'nochecksum' is honored in the plist file.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk
cp -p ${DISTMK} ${TREE}/usr/sbin/mkdist

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755 nochecksum
usr/sbin/mkdist root(0) root(0) 0755
EOF

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"

CWD=$(pwd)
PKG="test-1.0-1.$(uname -m).dist"

# create the package
create_pkg() {
    cmd="${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" -s sha256"
    [ -z "${LEAKCHECK}" ] && ${cmd} >/dev/null 2>&1 || ${LEAKCHECK} ${cmd}
}

# nochecksum
echo -n "checking 'nochecksum' support..."
create_pkg
cd ${OUTPUT}
tar -xf ${PKG} >/dev/null 2>&1
sum1="$(sqlite bom.db "SELECT digest FROM contents WHERE path='usr/sbin/distmk'")"
sum2="$(sqlite bom.db "SELECT digest FROM contents WHERE path='usr/sbin/mkdist'")"
computed="$(sha256sum ${TREE}/usr/sbin/mkdist | cut -d ' ' -f 1)"

if [ "${sum1}" = "*" -a "${sum2}" = "${computed}" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi

cd ${CWD}
rm -rf ${OUTPUT}/*

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected zero failures
[ ${FAILS} -gt 0 ] && exit 1 || exit 0
