#!/bin/sh
# Verify the required options on distmk are required.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

FAILS=0
TREE="$(mktemp -d)"

create_pkg() {
    [ -z "${LEAKCHECK}" ] && ${DISTMK} $* || ${LEAKCHECK} ${DISTMK} $*
}

# -t
echo -n "checking 'distmk -t'..."
out="$(create_pkg 2>&1)"
r=$?
echo "${out}" | grep -q "distmk: \*\*\* ERROR: Missing package root tree (-t)." >/dev/null 2>&1
a=$?
if [ ${r} -eq 1 ] && [ ${a} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + ${r}))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# -p
echo -n "checking 'distmk -p'..."
out="$(create_pkg -t ${TREE} 2>&1)"
r=$?
echo "${out}" | grep -q "distmk: \*\*\* ERROR: Missing package name (-p)." >/dev/null 2>&1
a=$?
if [ ${r} -eq 1 ] && [ ${a} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + ${r}))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# -v
echo -n "checking 'distmk -v'..."
out="$(create_pkg -t ${TREE} -p "test" 2>&1)"
r=$?
echo "${out}" | grep -q "distmk: \*\*\* ERROR: Missing package version (-v)." >/dev/null 2>&1
a=$?
if [ ${r} -eq 1 ] && [ ${a} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + ${r}))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# -r
echo -n "checking 'distmk -r'..."
out="$(create_pkg -t ${TREE} -p "test" -v "1.0" 2>&1)"
r=$?
echo "${out}" | grep -q "distmk: \*\*\* ERROR: Missing package release (-r)." >/dev/null 2>&1
a=$?
if [ ${r} -eq 1 ] && [ ${a} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + ${r}))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# -l
echo -n "checking 'distmk -l'..."
out="$(create_pkg -t ${TREE} -p "test" -v "1.0" -r 1 2>&1)"
r=$?
echo "${out}" | grep -q "distmk: \*\*\* ERROR: Missing path to plist file (-l)." >/dev/null 2>&1
a=$?
if [ ${r} -eq 1 ] && [ ${a} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + ${r}))
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

rm -rf ${TREE}

# expected 5 failures
[ ${FAILS} -eq 0 ] && exit 0 || exit 1
