#!/bin/sh
# Verify the options on distplist.

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

FAILS=0
TREE="$(mktemp -d)"
TMPDIR="$(mktemp -d)"

# plist package helper
plist_pkg() {
    [ -z "${LEAKCHECK}" ] && ${DISTPLIST} $* || ${LEAKCHECK} ${DISTPLIST} $*
}

# required -t option
echo -n "checking 'distplist -t'..."
out="$(plist_pkg 2>&1)"
r=$?
echo "${out}" | grep -q "*** ERROR: Missing package root tree (-t)." >/dev/null 2>&1
q=$?
if [ ${r} -eq 1 ] && [ ${q} -eq 0 ]; then
    FAILS=$((${FAILS} + ${r}))
    echo "passed."
else
    echo "FAILED."
fi
[ -z "${LEAKCHECK}" ] || echo "${out}"

# passing required -t option
SHORTFILE="$(basename "${DISTPLIST}")"
cp ${DISTPLIST} ${TREE}
echo -n "checking 'distplist -t PATH'..."
out="$(plist_pkg -t ${TREE})"
echo "${out}" | grep -v "^==" > ${TMPDIR}/plist
mode="$(stat -c %a ${TREE}/${SHORTFILE})"
[ $(echo ${mode} | wc -c) -eq 4 ] && mode="0${mode}"
echo "${SHORTFILE} $(id -un)($(id -u)) $(id -gn)($(id -g)) ${mode}" > ${TMPDIR}/plist.a
cmp -s ${TMPDIR}/plist ${TMPDIR}/plist.a
r=$?
FAILS=$((${FAILS} + ${r}))
[ ${r} -eq 1 ] && echo "FAILED." || echo "passed."
[ -z "${LEAKCHECK}" ] || echo "${out}"

# passing -u and -g options
SHORTFILE="$(basename "${DISTPLIST}")"
cp ${DISTPLIST} ${TREE}
echo -n "checking 'distplist -t PATH -u root -g root'..."
out="$(plist_pkg -t ${TREE} -u root -g root)"
echo "${out}" | grep -v "^==" > ${TMPDIR}/plist
mode="$(stat -c %a ${TREE}/${SHORTFILE})"
[ $(echo ${mode} | wc -c) -eq 4 ] && mode="0${mode}"
echo "${SHORTFILE} root(0) root(0) ${mode}" > ${TMPDIR}/plist.a
cmp -s ${TMPDIR}/plist ${TMPDIR}/plist.a
r=$?
FAILS=$((${FAILS} + ${r}))
[ ${r} -eq 1 ] && echo "FAILED." || echo "passed."
[ -z "${LEAKCHECK}" ] || echo "${out}"

rm -rf ${TREE}
rm -rf ${TMPDIR}

# expected 1 failure
[ ${FAILS} -ne 1 ] && exit 1 || exit 0
