#!/bin/sh
# Verify we can remove packages

CWD="$(pwd)"

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
EXPECTED_FAILS=0

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"
PKG1="test-1.0-1.$(uname -m).dist"
PKG2="test-2.0-1.$(uname -m).dist"

# create package helper
create_pkg() {
    if [ -z "${LEAKCHECK}" ]; then
        ${DISTMK} -t ${TREE} -p "test" -v "$1" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" >/dev/null 2>&1
    else
        ${LEAKCHECK} ${DISTMK} -t ${TREE} -p "test" -v "$1" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}"
    fi
}

# install package helper
install_pkg() {
    [ -z "${LEAKCHECK}" ] && env ETCMERGE=${ETCMERGE} ${DISTADD} $* || env ETCMERGE=${ETCMERGE} ${LEAKCHECK} ${DISTADD} $*
}

# remove package helper
remove_pkg() {
    [ -z "${LEAKCHECK}" ] && ${DISTRM} $* || ${LEAKCHECK} ${DISTRM} $*
}

# create a tree to package up
mkdir -p ${TREE}/usr/sbin
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

# create the package
create_pkg "1.0"

# create another package for test purposes
mv ${TREE}/usr/sbin/distmk ${TREE}/usr/sbin/distmk2

cat << EOF > ${META}/plist
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk2 root(0) root(0) 0755
EOF

create_pkg "2.0"

# remove a package
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distrm PKG'..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
out2="$(remove_pkg -r ${OUTPUT}/r test 2>&1)"
r=$?
lines=$(echo "${out2}" | grep -v "^==" | wc -l)
echo "${out2}" | grep -q "^Removing" >/dev/null 2>&1
q=$?
if [ ${r} -eq 0 ] && [ ${lines} -eq 1 ] && [ ${q} -eq 0 ] && [ ! -d ${OUTPUT}/r/usr ] && [ ! -d ${OUTPUT}/r/usr/sbin ] && [ ! -x ${OUTPUT}/r/usr/sbin/distmk ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# remove a package that does not exist (expect fail)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distrm PKG' on non-existent package..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
out2="$(remove_pkg -r ${OUTPUT}/r thing 2>&1)"
r=$?
echo "${out2}" | grep -q "No installed package matching 'thing' found." >/dev/null 2>&1
q=$?
if [ ${r} -eq 1 ] && [ ${q} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# remove a multi-version package (expect fail)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distrm PKG' on a multi-version package (expect fail)..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
out2="$(install_pkg -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
out3="$(remove_pkg -r ${OUTPUT}/r test 2>&1)"
r=$?
echo "${out3}" | grep -q "*** ERROR: More than one version of test found, please specify one to remove." >/dev/null 2>&1
q=$?
if [ ${r} -eq 1 ] && [ ${q} -eq 0 ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
    echo "${out3}"
fi

# remove a multi-version package (expect success)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'distrm PKG' on a multi-version package (expect success)..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG1} 2>&1)"
out2="$(install_pkg -d -m -r ${OUTPUT}/r ${OUTPUT}/${PKG2} 2>&1)"
out3="$(remove_pkg -r ${OUTPUT}/r test-1.0 2>&1)"
r=$?
lines=$(echo "${out3}" | grep -v "^==" | wc -l)
echo "${out3}" | grep -q "Removing" >/dev/null 2>&1
q=$?
if [ ${r} -eq 0 ] && [ ${lines} -eq 1 ] && [ ${q} -eq 0 ] && [ -x ${OUTPUT}/r/usr/sbin/distmk2 ] && [ ! -x ${OUTPUT}/r/usr/sbin/distmk ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
    echo "${out3}"
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected $EXPECTED_FAILS failures
[ ${FAILS} -eq ${EXPECTED_FAILS} ] && exit 0 || exit 1
