#!/bin/sh
# Verify etcmerge works as expected

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
EXPECTED_FAILS=0

# create package helper
create_pkg() {
    if [ -z "${LEAKCHECK}" ]; then
        ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" >/dev/null 2>&1
    else
        ${LEAKCHECK} ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}"
    fi
}

# install package helper
install_pkg() {
    [ -z "${LEAKCHECK}" ] && env ETCMERGE=${ETCMERGE} ${DISTADD} $* || env ETCMERGE=${ETCMERGE} ${LEAKCHECK} ${DISTADD} $*
}

# etcmerge package helper
etcmerge_pkg() {
    [ -z "${LEAKCHECK}" ] && ${ETCMERGE} $* || ${LEAKCHECK} ${ETCMERGE} $*
}

# create a tree to package up
mkdir -p ${TREE}/etc
mkdir -p ${TREE}/usr/sbin
echo "System ready" > ${TREE}/etc/issue
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
etc root(0) root(0) 0755
etc/issue root(0) root(0) 0644
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"
PKG="test-1.0-1.$(uname -m).dist"

# create the package
create_pkg

# initial configuration install
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'etcmerge -i' for initial config file install..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(etcmerge_pkg -i -r ${OUTPUT}/r 2>&1)"
r=$?
echo "${out2}" | grep "Installing new" | grep -q "/etc/issue" >/dev/null 2>&1
q=$?
if [ ${r} -eq 0 ] && [ ${q} -eq 0 ] && [ -r ${OUTPUT}/r/etc/issue ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# initial configuration install (pkgspec)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'etcmerge -i PKG' for initial config file install..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(etcmerge_pkg -i -r ${OUTPUT}/r test 2>&1)"
r=$?
echo "${out2}" | grep "Installing new" | grep -q "/etc/issue" >/dev/null 2>&1
q=$?
if [ ${r} -eq 0 ] && [ ${q} -eq 0 ] && [ -r ${OUTPUT}/r/etc/issue ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# query run (exit code indicates merge or not) single pkg - merge expected
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'etcmerge -q -i -m PKG' for single package..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(etcmerge_pkg -q -i -m -r ${OUTPUT}/r test 2>&1)"
r=$?
echo "${out2}" | grep "Installing new" | grep "/etc/issue" | grep -q "simulated" >/dev/null 2>&1
q=$?
if [ ${r} -eq 0 ] && [ ${q} -eq 0 ] && [ ! -r ${OUTPUT}/r/etc/issue ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# query run (exit code indicates merge or not) all pkgs - merge expected
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'etcmerge -q -i -m' for system..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(etcmerge_pkg -q -i -m -r ${OUTPUT}/r 2>&1)"
r=$?
echo "${out2}" | grep "Installing new" | grep "/etc/issue" | grep -q "simulated" >/dev/null 2>&1
q=$?
if [ ${r} -eq 0 ] && [ ${q} -eq 0 ] && [ ! -r ${OUTPUT}/r/etc/issue ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# query run (exit code indicates merge or not) all pkgs - no merge expected
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'etcmerge -q -i -m' for system (no merge expected)..."
out1="$(install_pkg -d -e -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(etcmerge_pkg -q -i -m -r ${OUTPUT}/r 2>&1)"
r=$?
echo "${out2}" | grep "Installing new" | grep "/etc/issue" | grep -q "simulated" >/dev/null 2>&1
q=$?
if [ ${r} -eq 1 ] && [ ${q} -eq 1 ] && [ -r ${OUTPUT}/r/etc/issue ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# honor /etc/dist/install.list
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r/etc/dist
echo "other string" > ${OUTPUT}/r/etc/issue
echo "/etc/issue" > ${OUTPUT}/r/etc/dist/install.list
echo -n "checking that etcmerge honors /etc/dist/install.list..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(etcmerge_pkg -i -r ${OUTPUT}/r test 2>&1)"
r=$?
echo "${out2}" | grep -q "/etc/issue already exists, overwriting" >/dev/null 2>&1
q=$?
if [ ${r} -eq 0 ] && [ ${q} -eq 0 ] && [ "$(cat ${OUTPUT}/r/etc/issue)" = "System ready" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# ignore /etc/dist/install.list
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r/etc/dist
echo "other string" > ${OUTPUT}/r/etc/issue
echo "/etc/issue" > ${OUTPUT}/r/etc/dist/install.list
echo -n "checking that 'etcmerge -I' ignores /etc/dist/install.list..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(etcmerge_pkg -I -i -m -q -r ${OUTPUT}/r test 2>&1)"
r=$?
echo "${out2}" | grep "Installing new" | grep "/etc/issue" | grep -q "simulated" >/dev/null 2>&1
q=$?
if [ ${r} -eq 0 ] && [ ${q} -eq 0 ] && [ "$(cat ${OUTPUT}/r/etc/issue)" = "other string" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# honor /etc/dist/skip.list
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r/etc/dist
echo "other string" > ${OUTPUT}/r/etc/issue
echo "/etc/issue" > ${OUTPUT}/r/etc/dist/skip.list
echo -n "checking that etcmerge honors /etc/dist/skip.list..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(etcmerge_pkg -i -r ${OUTPUT}/r test 2>&1)"
r=$?
echo "${out2}" | grep -q "/etc/issue" >/dev/null 2>&1
q=$?
if [ ${r} -eq 0 ] && [ ${q} -eq 1 ] && [ "$(cat ${OUTPUT}/r/etc/issue)" = "other string" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

# ignore /etc/dist/skip.list
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r/etc/dist
echo "other string" > ${OUTPUT}/r/etc/issue
echo "/etc/issue" > ${OUTPUT}/r/etc/dist/skip.list
echo -n "checking that 'etcmerge -S' ignores /etc/dist/skip.list..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(${ETCMERGE} -S -i -m -q -r ${OUTPUT}/r test 2>&1)"
r=$?
echo "${out2}" | grep "Installing new" | grep "/etc/issue" | grep -q "simulated" >/dev/null 2>&1
q=$?
if [ ${r} -eq 0 ] && [ ${q} -eq 0 ] && [ "$(cat ${OUTPUT}/r/etc/issue)" = "other string" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected $EXPECTED_FAILS failures
[ ${FAILS} -eq ${EXPECTED_FAILS} ] && exit 0 || exit 1
