#!/bin/sh
# Verify etcorphan works as expected

title=" >>> $(basename ${0}) <<< "
line="-$(echo ${title} | sed -e 's|.|-|g')-"
echo
echo "+${line}+"
echo "|${title}|"
echo "+${line}+"
echo

TREE="$(mktemp -d)"
META="$(mktemp -d)"
OUTPUT="$(mktemp -d)"
TMPDIST="$(mktemp -d)"
FAILS=0
EXPECTED_FAILS=0

# create package helper
create_pkg() {
    if [ -z "${LEAKCHECK}" ]; then
        ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}" >/dev/null 2>&1
    else
        ${LEAKCHECK} ${DISTMK} -t ${TREE} -p "test" -v "1.0" -r 1 -m ${META} -l ${META}/plist -R peachtree -o ${OUTPUT} --tmpdir=${TMPDIST} -D "${DESC}" -M "${MAINTAINER}" -L "${LICENSE}"
    fi
}

# install package helper
install_pkg() {
    [ -z "${LEAKCHECK}" ] && env ETCMERGE=${ETCMERGE} ${DISTADD} $* || env ETCMERGE=${ETCMERGE} ${LEAKCHECK} ${DISTADD} $*
}

# etcmerge package helper
etcmerge_pkg() {
    [ -z "${LEAKCHECK}" ] && ${ETCMERGE} $* || ${LEAKCHECK} ${ETCMERGE} $*
}

# etcorphan package helper
etcorphan_pkg() {
    [ -z "${LEAKCHECK}" ] && ${ETCORPHAN} $* || ${LEAKCHECK} ${ETCORPHAN} $*
}

# create a tree to package up
mkdir -p ${TREE}/etc
mkdir -p ${TREE}/usr/sbin
echo "System ready" > ${TREE}/etc/issue
cp -p ${DISTMK} ${TREE}/usr/sbin/distmk

# create plist
cat << EOF > ${META}/plist
etc root(0) root(0) 0755
etc/issue root(0) root(0) 0644
usr root(0) root(0) 0755
usr/sbin root(0) root(0) 0755
usr/sbin/distmk root(0) root(0) 0755
EOF

DESC="Test program"
MAINTAINER="J. Random Packager <someone@example.net>"
LICENSE="WTFPL"
PKG="test-1.0-1.$(uname -m).dist"

# create the package
create_pkg

# report owned files (via single file specification and -r usage)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'etcorphan FILE' for initial config file install..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(etcmerge_pkg -i -r ${OUTPUT}/r 2>&1)"
out3="$(etcorphan -r ${OUTPUT}/r /etc/issue 2>&1)"
r=$?
out4="$(echo "${out3}" | grep "/etc/issue:" | cut -d ' ' -f 2)"
if [ ${r} -eq 0 ] && [ "${out4}" = "$(basename ${PKG} .dist)" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
    echo "${out3}"
fi

# report unowned files (with no unowned files existing)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'etcorphan' for unowned file reporting (w/ no unowned files)..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(etcmerge_pkg -i -r ${OUTPUT}/r 2>&1)"
out3="$(etcorphan -r ${OUTPUT}/r 2>&1)"
r=$?
out4="$(echo "${out3}" | grep -v "^==")"
if [ ${r} -eq 0 ] && [ -z "${out4}" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
    echo "${out3}"
fi

# report unowned files (with one unowned files existing)
rm -rf ${OUTPUT}/r
mkdir -p ${OUTPUT}/r
echo -n "checking 'etcorphan' for unowned file reporting (w/ one unowned file)..."
out1="$(install_pkg -d -r ${OUTPUT}/r ${OUTPUT}/${PKG} 2>&1)"
out2="$(etcmerge_pkg -i -r ${OUTPUT}/r 2>&1)"
echo "example" > ${OUTPUT}/r/etc/example.conf
out3="$(etcorphan -r ${OUTPUT}/r 2>&1)"
r=$?
out4="$(echo "${out3}" | grep -v "^==")"
if [ ${r} -eq 0 ] && [ ! -z "${out4}" ] && [ "${out4}" = "/etc/example.conf" ]; then
    echo "passed."
else
    FAILS=$((${FAILS} + 1))
    echo "FAILED."
fi
if [ ! -z "${LEAKCHECK}" ]; then
    echo "${out1}"
    echo "${out2}"
    echo "${out3}"
fi

rm -rf ${TREE}
rm -rf ${META}
rm -rf ${TMPDIST}
rm -rf ${OUTPUT}

# expected $EXPECTED_FAILS failures
[ ${FAILS} -eq ${EXPECTED_FAILS} ] && exit 0 || exit 1
