#!/bin/sh
#
# clean-chroot.sh: Clean the $CHROOT path and preserve some things
#
# Copyright David Cantrell <dcantrell@burdell.org>
# SPDX-License-Identifier: Apache-2.0

# This script returns zero on success, non-zero on error.
# The following environment variables need to be set for the script:
#    $CHROOT               The chroot path (cannot be '/').
# This script is called by the build system scripts and should not be called
# directly.

if [ -z "${CHROOT}" ]; then
    echo "*** You must specify the CHROOT= variable." >&2
    exit 1
fi

if [ -d "${CHROOT}" ] && [ "$(readlink -f "${CHROOT}")" = "/" ]; then
    exit 2
fi

if [ "$(id -u)" -ne 0 ]; then
    echo "*** You must be root to perform a chroot clean." >&2
    exit 3
fi

for subdir in bin boot etc lib mnt run root sbin tmp usr var ; do
    [ -d "${CHROOT}"/"${subdir}" ] || continue
    printf "%s" "Removing ${CHROOT}/${subdir}..."
    rm -rf "${CHROOT:?}"/"${subdir}" || exit 4
    printf "%s" "done."
done

exit 0
