#!/bin/sh
#
# incver.sh: Increment DISTRIBUTION_VER and reset all package REL
#            counts to 1 for packages versioned after the
#            DISTRIBUTION_VER.
#
# Copyright David Cantrell <dcantrell@burdell.org>
# SPDX-License-Identifier: Apache-2.0

# This script returns zero on success, non-zero on error.
# The following environment variables need to be set for the script:
#    $TOPSRC               The top level project directory.
# This script is called by the build system scripts and should not be called
# directly.

PATH=/usr/bin

if [ -z "${TOPSRC}" ] || [ ! -d "${TOPSRC}" ]; then
    exit 1
fi

# Increment the DISTRIBUTION_VERSION value
v="$(grep ^DISTRIBUTION_VERSION "${TOPSRC}"/base/system/dist-tools/src/include/pkg.mk | rev | cut -d ' ' -f 1 | rev)"
major="$(echo "${v}" | cut -d '.' -f 1)"
minor="$(echo "${v}" | cut -d '.' -f 2)"
update="$(echo "${v}" | cut -d '.' -f 3)"
newminor=$((minor + 1))
sed -i -e "s|^DISTRIBUTION_VERSION\ =\ .*\$|DISTRIBUTION_VERSION = ${major}.${newminor}.${update}|g" "${TOPSRC}"/base/system/dist-tools/src/include/pkg.mk

# Write out new make.conf
echo "NAME = Peachtree Linux" > "${TOPSRC}"/mk/make.conf
echo "VERSION = ${major}.${newminor}.${update}" >> "${TOPSRC}"/mk/make.conf

# Reset all REL values in packages that use DISTRIBUTION_VERSION
find "${TOPSRC}"/ -type f -name Makefile -print0 | xargs -0 grep "VER =" | grep DISTRIBUTION_VERSION | cut -d ':' -f 1 | \
while read -r f ; do
    sed -i -e "s|^REL\ =\ .*\$|REL = 1|g" "${f}"
done
