#!/bin/sh
#
# merge-bundles.sh: Takes bundles from the specified incoming directory
#                   and merges them with the specified published release
#                   tree.  This script is invoked by the Buildbot system.
#
# Copyright David Cantrell <dcantrell@burdell.org>
# SPDX-License-Identifier: Apache-2.0

# This script returns zero on success, non-zero on error.
# The environment variables used come from Buildbot.
# This script is called by the build system scripts and should not be called
# directly.

[ -z "${INCOMING_DIR}" ] && exit 1
[ -z "${BRANCH}" ] && exit 2

NO_RELEASE=0
NO_UPDATES=0

if [ -z "${RELEASE_DIR}" ] || [ ! -d "${RELEASE_DIR}" ]; then
    NO_RELEASE=1
fi

if [ -z "${UPDATES_DIR}" ] || [ ! -d "${UPDATES_DIR}" ]; then
    NO_UPDATES=1
fi

if [ ${NO_RELEASE} -eq ${NO_UPDATES} ]; then
    exit 3
fi

if [ "${BRANCH}" = "master" ] && [ ${NO_RELEASE} -eq 1 ]; then
    exit 4
fi

if [ ! "${BRANCH}" = "master" ] && [ ${NO_UPDATES} -eq 1 ]; then
    exit 5
fi

TMPDIR="$(mktemp -d)"
trap 'rm -rf "${TMPDIR}"' EXIT

# Process bundles for this branch
[ -d "${HOME}/${INCOMING_DIR}/${BRANCH}" ] || exit 0

BUNDLES="$(ls -1 "${HOME}"/"${INCOMING_DIR}"/"${BRANCH}"/*.bundle 2>/dev/null)"

if [ -z "${BUNDLES}" ]; then
    echo "No bundles to merge."
    exit 0
fi

for bundle in ${BUNDLES} ; do
    cd "${TMPDIR}" || exit 1

    bundle_subdir="$(basename "${bundle}" .bundle)"
    tar -xvf "${bundle}"
    cd "${bundle_subdir}" || exit 1

    # merge in to the correct branch
    # XXX: add in more branch handling here
    if [ "${BRANCH}" = "master" ]; then
        ./merge.sh "${RELEASE_DIR}/devel"
    elif [ "${BRANCH}" = "3.5-stable" ]; then
        ./merge.sh "${UPDATES_DIR}"
        makerepo "${UPDATES_DIR}"
    fi

    cd "${TMPDIR}" || exit 1
    rm -f "${bundle}"
done

# Clean up and exit
exit 0
