#!/bin/sh
#
# show-leafs.sh: Show leaf packages; not required by anything else
#
# Copyright David Cantrell <dcantrell@burdell.org>
# SPDX-License-Identifier: Apache-2.0

# This script returns zero on success, non-zero on error.
# The following environment variables need to be set for the script:
#    $TOPSRC               The top level distribution source directory.
# This script is called by the build system scripts and should not be called
# directly.

PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin

if [ -z "${TOPSRC}" ] || [ ! -d "${TOPSRC}" ]; then
    exit 1
fi

TMPDIR="$(mktemp -d)"
PKGLIST="${TMPDIR}"/pkglist
REQLIST="${TMPDIR}"/reqlist

# Build lists
find "${TOPSRC}"/ -type f -name "plist*" | grep -vE 'plist.*\.[cho5]$' | while read -r pkgdir ; do
    subdir="$(dirname "${pkgdir}")"

    # package source tree name
    echo "${subdir}" | sed -e "s|^${TOPSRC}/||g" >> "${PKGLIST}"

    # buildreqs
    if [ -f "${subdir}"/buildreqs ]; then
        cat "${subdir}"/buildreqs  >> "${REQLIST}"
    fi
done

# Sort and uniq the lists
sort "${PKGLIST}" | uniq > "${PKGLIST}".new
mv "${PKGLIST}".new "${PKGLIST}"
sort "${REQLIST}" | uniq > "${REQLIST}".new
mv "${REQLIST}".new "${REQLIST}"

# Show the leaf packages
comm -23 "${PKGLIST}" "${REQLIST}"

# Clean up
rm -rf "${TMPDIR}"
exit 0
