# Mirror targets
#
# Copyright David Cantrell <dcantrell@burdell.org>
# SPDX-License-Identifier: Apache-2.0

-include <pkg.mk>

HELP += "    mirrors              Run all mirror-* targets\n"

mirrors: mirror-distfiles mirror-devel

# ftp://ftp.burdell.org/pub/peachtree/distfiles/
DISTFILES_HOST = kevlar.burdell.org
DISTFILES_PATH = /srv/ftp/pub/peachtree/distfiles/

HELP += "   mirror-distfiles     Upload distfiles to $(DISTFILES_HOST)\n"

# ftp://ftp.burdell.org/pub/peachtree/devel/
DEVEL_HOST = kevlar.burdell.org
DEVEL_PATH = /srv/ftp/pub/peachtree/devel/

HELP += "   mirror-devel         Upload devel release tree to $(DEVEL_HOST)\n"

# The current stable release
RELEASE_HOST = kevlar.burdell.org
RELEASE_PATH = /srv/ftp/pub/peachtree/$(DISTRIBUTION_VERSION)/

ifeq ($(TARGET_MACH),x86_64)
RELEASE_EXCLUDE = --exclude ppc64
endif

ifeq ($(TARGET_MACH),ppc64)
RELEASE_EXCLUDE = --exclude x86_64
endif

HELP += "   pull                 Download current stable release to $(RELEASE)\n"
HELP += "   pull-devel           Download current development release to $(RELEASE)\n"

# Some options are only available in later rsync protocol versions
RSYNC30_OPTS = -rclOIvz --no-o --no-g --rsh=ssh --progress --delete
RSYNC31_OPTS = $(RSYNC30_OPTS) -J

# This will only work if the current user has 'kevlar.burdell.org' access
# configured in their ~/.ssh/config file.
#
# NOTE:  Common options are -avz, but we use these options:
#            -r    Recursive
#            -l    Symlinks are symlinks, do not dereference
#            -c    Skip based on checksum, not mtime+size
#            -t    Preserve timestamps
#            -O    Omit directories when preserving timestamps (only
#                  works if you are the remote owner of the dir, group
#                  write access is not sufficient)
#            -J    Same as -O, but for symlinks (same reason)
#            -I    Ignore timestamps since those may be all over the place
#            -v    Verbose
#            -z    Compress the network traffic
mirror-distfiles:
	@if [ -d $(DISTFILES) ]; then \
		if [ "$(DISTFILES_HOST)" = "$$(hostname)" ]; then \
			mkdir -p $(DISTFILES_PATH) ; \
			dest="$(DISTFILES_PATH)" ; \
		else \
			ssh $(DISTFILES_HOST) mkdir -p $(DISTFILES_PATH) ; \
			dest="$(DISTFILES_HOST):$(DISTFILES_PATH)" ; \
		fi ; \
		rsync $(RSYNC31_OPTS) $(DISTFILES)/ $${dest} ; \
		if [ $$? -eq 12 ]; then \
			rsync $(RSYNC30_OPTS) $(DISTFILES)/ $${dest} ; \
		fi ; \
	fi

mirror-devel:
	@if [ -d $(RELEASE) ]; then \
		ssh $(DEVEL_HOST) mkdir -p $(DEVEL_PATH) ; \
		rsync $(RSYNC31_OPTS) $(RELEASE)/ $(DEVEL_HOST):$(DEVEL_PATH) ; \
		if [ $$? -eq 12 ]; then \
			rsync $(RSYNC30_OPTS) $(RELEASE)/ $(DEVEL_HOST):$(DEVEL_PATH) ; \
		fi ; \
	fi

pull:
	[ -d $(RELEASE) ] || $(SUDO) mkdir -p $(RELEASE)
	rsync $(RSYNC31_OPTS) $(RELEASE_EXCLUDE) $(RELEASE_HOST):$(RELEASE_PATH) $(RELEASE)/ ; \
	if [ $$? -eq 12 ]; then \
		rsync $(RSYNC30_OPTS) $(RELEASE_EXCLUDE) $(RELEASE_HOST):$(RELEASE_PATH) $(RELEASE)/ ; \
	fi

# This pulls from devel, but stores in in the distribution version release
# directory.
pull-devel:
	releasepath="$$(echo $(RELEASE_PATH) | sed -e 's|$(DISTRIBUTION_VERSION)|devel|g')" ; \
	[ -d $(RELEASE) ] || $(SUDO) mkdir -p $(RELEASE) ; \
	rsync $(RSYNC31_OPTS) $(RELEASE_EXCLUDE) $(RELEASE_HOST):$${releasepath} $(RELEASE)/ ; \
	if [ $$? -eq 12 ]; then \
		rsync $(RSYNC30_OPTS) $(RELEASE_EXCLUDE) $(RELEASE_HOST):$${releasepath} $(RELEASE)/ ; \
	fi
