#!/bin/sh
#
# Make sure our $CFLAGS are not in any /usr/bin scripts
#

PATH=/usr/bin:/usr/sbin
CWD="$(pwd)"
TEST="$0"
LOGNAME="$(basename $TEST .sh).log"
LOG="$CWD/$LOGNAME"
EXITSTATUS=0

rm -f $LOG

_check_tree() {
    localstatus=0
    echo -n "checking for CFLAGS in $1 scripts..."

    if [ ! -d $1 ]; then
        echo "directory not present [skipping]"
        return
    fi

    find $1 -type f | \
    grep -vE "(hardening-check|python3.*-config)" 2>/dev/null | \
    while read fn ; do
        if grep -q -E "(fstack-protector-strong|D_FORTIFY_SOURCE|mtune)" "$fn" >/dev/null 2>&1 ; then
            grep -E "(fstack-protector-strong|D_FORTIFY_SOURCE|mtune)" >> $LOG 2>/dev/null
            [ $EXITSTATUS -eq 0 ] && EXITSTATUS=1
            [ $localstatus -eq 0 ] && localstatus=1
        fi
    done

    if [ $localstatus -eq 1 ]; then
        echo "FAIL (see $LOGNAME)"
    else
        echo "pass"
    fi
}

echo "Command:" >> $LOG
echo "_check_tree /usr/bin" >> $LOG
echo "_check_tree /usr/sbin" >> $LOG
echo "_check_tree /usr/X11/bin" >> $LOG
echo "_check_tree /usr/X11/sbin" >> $LOG
echo "_check_tree /usr/wayland/bin" >> $LOG
echo "_check_tree /usr/wayland/sbin" >> $LOG
echo >> $LOG

_check_tree /usr/bin
_check_tree /usr/sbin
_check_tree /usr/X11/bin
_check_tree /usr/X11/sbin
_check_tree /usr/wayland/bin
_check_tree /usr/wayland/sbin

[ $EXITSTATUS -eq 0 ] && rm -f $LOG
exit $EXITSTATUS
