#!/bin/sh
#
# Verify there are no misplaced cmake files
#

PATH=/usr/bin:/usr/sbin
CWD="$(pwd)"
TEST="$0"
LOGNAME="$(basename $TEST .sh).log"
LOG="$CWD/$LOGNAME"
TMPDIR="$(mktemp -d)"
RET=0
trap 'rm -rf ${TMPDIR}' EXIT

rm -f $LOG

echo -n "checking for misplaced cmake files in /usr..."

find /usr -not \( -path /usr/src -prune \) \
          -not \( -path /usr/obj -prune \) \
          -not \( -path /usr/distfiles -prune \) \
          -not \( -path /usr/release -prune \) \
          -not \( -path /usr/share/doc -prune \) \
          -not \( -path /usr/share/licenses -prune \) \
          -not \( -path /usr/X11 -prune \) \
          -not \( -path /usr/wayland -prune \) \
          -type f -name "*.cmake" | \
grep -vE "/usr/(share|lib)/cmake" | \
grep -vE "/usr/lib/python.*/site-packages/mesonbuild/cmake/data/preload.cmake" > $TMPDIR/bad-cmake-files
if [ $? -eq 0 ]; then
    echo "FAIL (see $LOGNAME)"

    echo "Command:" >> $LOG
    echo "find /usr -type f -name \"*.cmake\" | \\" >> $LOG
    echo "    grep -vE \"/usr/(share|lib)/cmake\" | \\" >> $LOG
    echo "    grep -vE \"/usr/lib/python.*/site-packages/mesonbuild/cmake/data/preload.cmake\"" >> $LOG
    echo >> $LOG
    while read line ; do
        echo "$line is not in /usr/share/cmake or /usr/lib/cmake" >> $LOG
    done < $TMPDIR/bad-cmake-files
    RET=$(($RET + 1))
else
    echo "pass"
fi

echo -n "checking for misplaced cmake files in /usr/X11..."

find /usr/X11 -type f -name "*.cmake" | \
grep -vE "/usr/X11/(share|lib)/cmake" > $TMPDIR/bad-cmake-files
if [ $? -eq 0 ]; then
    echo "FAIL (see $LOGNAME)"

    echo "Command:" >> $LOG
    echo "find /usr/X11 -type f -name \"*.cmake\" | grep -vE \"/usr/X11/(share|lib)/cmake\"" >> $LOG
    echo >> $LOG
    while read line ; do
        echo "$line is not in /usr/X11/share/cmake or /usr/X11/lib/cmake" >> $LOG
    done < $TMPDIR/bad-cmake-files
    RET=$(($RET + 1))
else
    echo "pass"
fi

echo -n "checking for misplaced cmake files in /usr/wayland..."

find /usr/wayland -type f -name "*.cmake" | \
grep -vE "/usr/wayland/(share|lib)/cmake" > $TMPDIR/bad-cmake-files
if [ $? -eq 0 ]; then
    echo "FAIL (see $LOGNAME)"

    echo "Command:" >> $LOG
    echo "find /usr/wayland -type f -name \"*.cmake\" | grep -vE \"/usr/wayland/(share|lib)/cmake\"" >> $LOG
    echo >> $LOG
    while read line ; do
        echo "$line is not in /usr/wayland/share/cmake or /usr/wayland/lib/cmake" >> $LOG
    done < $TMPDIR/bad-cmake-files
    RET=$(($RET + 1))
else
    echo "pass"
fi

exit $RET
