#!/bin/sh
#
# Verify ELF library files are all 0755
#

PATH=/usr/bin:/usr/sbin
CWD="$(pwd)"
TEST="$0"
TMPDIR="$(mktemp -d)"
LOGNAME="$(basename $TEST .sh).log"
LOG="$CWD/$LOGNAME"
RET=0

rm -f $LOG
echo -n "checking all ELF shared objects have correct permissions..."

find /usr/lib -name "*.so*" -type f > $TMPDIR/sofiles

while read fname ; do
    [ -L "$fname" ] && return
    t="$(file -b --mime-type "$fname")"

    if [ "$t" = "application/x-sharedlib" ]; then
        perms="$(stat -c %a "$fname")"

        if [ ! "$perms" = "755" ]; then
            echo "$fname does not have 755 permissions" >> $LOG
            RET=$(($RET + 1))
        fi
    fi
done < $TMPDIR/sofiles

[ $RET -eq 0 ] && echo "pass" || echo "FAIL"

rm -rf $TMPDIR
exit $RET
