#!/bin/sh
#
# Make sure /usr/games and /usr/X11/games files are owned by group
# games.
#

PATH=/usr/bin:/usr/sbin
CWD="$(pwd)"
RET=0

_check_games_dir() {
    dname="${1}"
    reqgroup="${2}"

    [ -d "${dname}" ] || return

    gname="$(stat -c %G ${dname})"
    if [ ! "${gname}" = "${reqgroup}" ]; then
        echo "Invalid group ownership for ${dname}:  ${gname}"
        RET=$((${RET} + 1))
    fi

    find ${dname} -type f | while read pathname ; do
        gname="$(stat -c %G ${pathname})"
        if [ ! "${gname}" = "games" ]; then
            echo "Invalid group ownership for ${pathname}:  ${gname}"
            RET=$((${RET} + 1))
        fi
    done
}

_check_group_write() {
    dname="${1}"

    [ -d "${dname}" ] || return

    gwrite="$(stat -c %A ${dname} | cut -c6)"
    if [ ! "${gwrite}" = "w" ]; then
        echo "Missing group write permissions for ${dname}"
        RET=$((${RET} + 1))
    fi

    find ${dname} | while read pathname ; do
        gwrite="$(stat -c %A ${pathname} | cut -c6)"
        if [ ! "${gwrite}" = "w" ]; then
            echo "Missing group write permissions for ${pathname}"
            RET=$((${RET} + 1))
        fi
    done
}

_check_games_dir /usr/games games
_check_games_dir /usr/X11/games games
_check_games_dir /usr/wayland/games games
_check_games_dir /var/lib/games games

_check_group_write /var/lib/games

exit ${RET}
