#!/bin/sh
#
# Verify ELF executables and libraries are built with hardened compiler
# flags and C library options.
#

PATH=/usr/bin:/usr/sbin
CWD="$(pwd)"
TEST="$0"
LOGNAME="$(basename $TEST .sh).log"
LOG="$CWD/$LOGNAME"
RET=0

rm -f $LOG
echo -n "checking for hardened builds..."

_gather_files() {
    find /usr \
         -not \( -path /usr/src -prune \) \
         -not \( -path /usr/obj -prune \) \
         -not \( -path /usr/distfiles -prune \) \
         -not \( -path /usr/release -prune \) \
         -not \( -path /usr/lib/gconv -prune \) \
         -not \( -path /usr/lib/ghc -prune \) \
         -type f | \
    grep -v "^/usr/lib/ghc"
}

_gather_files | while read fname ; do
    [ -f "$fname" ] || continue
    t="$(file -b --mime-type "$fname")"

    if [ "$t" = "application/x-executable" -o "$t" = "application/x-sharedlib" ]; then
        # Skip 32-bit content, which is likely special emulator or boot
        # loader files.
        file "$fname" | grep -q "ELF 32-bit" >/dev/null 2>&1
        [ $? -eq 0 ] && continue

        # Skip Haskell programs
        ldd "$fname" | grep -q libHS >/dev/null 2>&1
        [ $? -eq 0 ] && continue

        hardening-check -p -q -v "$fname" >> $LOG
        [ $? -ne 0 ] && echo >> $LOG
    fi
done

grep -q 'found!' $LOG >/dev/null 2>&1
RET=$?

[ $RET -eq 1 ] && echo "pass" || echo "FAIL (see $LOGNAME)"

exit $RET
