#!/bin/sh
#
# Verify the linker scripts in /usr/lib are correct.
# They should not have directory paths, just filenames.
#

PATH=/usr/bin:/usr/sbin
CWD="$(pwd)"
TEST="$(basename $0)"
LOG="$CWD/$(basename $TEST .sh).log"

rm -f $LOG

check_linker_script() {
    scriptpath="/usr/lib/$1"

    echo -n "checking $scriptpath linker script..."

    if [ ! -f $scriptpath ]; then
        echo "FAIL...$scriptpath does not exist." >&2
        exit 1
    fi

    CMD="grep '/' $scriptpath | grep -vE '(\/\*|\*\/)'"

    echo "Command:" >> $LOG
    echo "$CMD" >> $LOG
    echo >> $LOG
    echo "$TEST $scriptpath" >> $LOG
    echo "$(echo "$TEST $scriptpath" | sed -e 's|.|=|g')" >> $LOG
    echo >> $LOG

    eval $CMD >/dev/null 2>&1
    retval=$?

    if [ $retval -eq 0 ]; then
        echo "FAIL (see $LOG)"
        $CMD >> $LOG 2>&1
        exit $retval
    else
        echo "pass" | tee -a $LOG
    fi
}

check_linker_script libc.so
[ "$(uname -m)" = "ppc64" ] || check_linker_script libm.so

rm -f $LOG
exit 0
