#!/bin/sh
#
# Verify all D-Bus files lack settings for systemd
#

PATH=/usr/bin:/usr/sbin
CWD="$(pwd)"
TEST="$0"
TMPDIR="$(mktemp -d)"
LOGNAME="$(basename $TEST .sh).log"
LOG="$CWD/$LOGNAME"
RET=0

trap 'rm -rf ${TMPDIR}' EXIT

rm -f ${LOG}

for d in /usr/share /usr/X11/share ; do
    [ -d ${d}/dbus-1 ] || continue

    echo "y" > ${TMPDIR}/pass
    echo -n "checking for forbidden systemd references in ${d}/dbus-1..."

    find ${d}/dbus-1 -type f | while read f ; do
        # this file mentions systemd, but not in a harmful way
        [ "$f" = "/usr/share/dbus-1/system.conf" ] && continue

        grep -q -i systemd "${f}" >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            echo "n" > ${TMPDIR}/pass
            RET=$((${RET} + 1))
            grep -H -i systemd "${f}" >> ${LOGNAME}
        fi
    done

    [ "$(cat ${TMPDIR}/pass 2>/dev/null)" = "y" ] && echo "pass" || echo "FAIL"
done

exit ${RET}
