#!/bin/sh
#
# Verify all Wayland linked files live in /usr/wayland
#

PATH=/usr/bin:/usr/sbin
CWD="$(pwd)"
TEST="$0"
TMPDIR="$(mktemp -d)"
LOGNAME="$(basename $TEST .sh).log"
LOG="$CWD/$LOGNAME"
RET=0

rm -f $LOG
echo -n "checking all Wayland-linked ELF shared objects live in /usr/wayland..."

find /usr -maxdepth 1 | \
    grep -v '^/usr$' | \
    grep -vE "(wayland|X11|lib64|distfiles|debug|src|release|obj|local)" | \
while read dname ; do
    find $dname -type f >> $TMPDIR/filelist
done

sort $TMPDIR/filelist | uniq > $TMPDIR/filelist.n
mv $TMPDIR/filelist.n $TMPDIR/filelist

# XXX: filter out things that we cannot move yet
grep -vE "(-ghc|ghc-)" $TMPDIR/filelist | grep -vE "^/usr/lib/java" > $TMPDIR/filelist.n
mv $TMPDIR/filelist.n $TMPDIR/filelist

while read fname ; do
    [ -L "$fname" ] && continue
    t="$(file -b --mime-type "$fname")"

    if [ "$t" = "application/x-sharedlib" ] || [ "$t" = "application/x-executable" ]; then
        # /usr/bin/[ is a real command that messes up grep commands
        gfname="$(echo "$fname" | sed -e 's|\[|\\[|g')"

        ldd -v $fname 2>/dev/null | grep -E "/.*:" | cut -d ':' -f 1 | grep -v "$gfname" | grep -q "/usr/wayland" >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            echo "$fname links with Wayland files" >> $LOG
            RET=$(($RET + 1))
        fi
    fi
done < $TMPDIR/filelist

[ $RET -eq 0 ] && echo "pass" || echo "FAIL"

rm -rf $TMPDIR
exit $RET
