#!/bin/sh
#
# Try to find any misplaced pkg-config scripts.
#

PATH=/usr/bin:/usr/sbin
CWD="$(pwd)"
TEST="$0"
LOGNAME="$(basename $TEST .sh).log"
LOG="$CWD/$LOGNAME"
RET=0

rm -f $LOG

_check_usr() {
    echo -n "checking for any misplaced pkg-config scripts /usr/lib/pkgconfig..."
    find /usr -not \( -path /usr/src -prune \) \
              -not \( -path /usr/obj -prune \) \
              -not \( -path /usr/distfiles -prune \) \
              -not \( -path /usr/release -prune \) \
              -not \( -path /usr/share/doc -prune \) \
              -not \( -path /usr/share/licenses -prune \) \
              -not \( -path /usr/X11 -prune \) \
              -not \( -path /usr/wayland -prune \) \
              -type f \
              -name "*.pc" | \
    grep -v "/usr/lib/pkgconfig"
    return $?
}

_check_usr_X11() {
    echo -n "checking for any misplaced pkg-config scripts in /usr/X11/lib/pkgconfig..."
    find /usr/X11 -type f -name "*.pc" | grep -v "/usr/X11/lib/pkgconfig"
    return $?
}

_check_usr_wayland() {
    echo -n "checking for any misplaced pkg-config scripts in /usr/wayland/lib/pkgconfig..."
    find /usr/wayland -type f -name "*.pc" | grep -v "/usr/wayland/lib/pkgconfig"
    return $?
}

_check_usr 2>&1
if [ $? -eq 0 ]; then
    echo "FAIL (see $LOGNAME)"

    echo "Command:" >> $LOG
    echo 'find /usr -not \( -path /usr/src -prune \) \' >> $LOG
    echo '          -not \( -path /usr/obj -prune \) \' >> $LOG
    echo '          -not \( -path /usr/distfiles -prune \) \' >> $LOG
    echo '          -not \( -path /usr/release -prune \) \' >> $LOG
    echo '          -not \( -path /usr/share/doc -prune \) \' >> $LOG
    echo '          -not \( -path /usr/share/licenses -prune \) \' >> $LOG
    echo '          -not \( -path /usr/X11 -prune \) \' >> $LOG
    echo '          -not \( -path /usr/wayland -prune \) \' >> $LOG
    echo '          -type f \' >> $LOG
    echo '          -name "*.pc" | \' >> $LOG
    echo 'grep -v "/usr/lib/pkgconfig"' >> $LOG
    echo >> $LOG
    echo "$TEST" >> $LOG
    echo "$(echo $TEST | sed -e 's|.|=|g')" >> $LOG
    echo >> $LOG

    _check_usr | while read filename ; do
        distinfo -q $filename >> $LOG 2>&1
    done

    RET=$(($RET + 1))
else
    echo "pass"
fi

_check_usr_X11 2>&1
if [ $? -eq 0 ]; then
    echo "FAIL (see $LOGNAME)"

    echo "Command:" >> $LOG
    echo 'find /usr/X11 -type f -name "*.pc" | grep -v "/usr/X11/lib/pkgconfig"' >> $LOG
    echo >> $LOG
    echo "$TEST" >> $LOG
    echo "$(echo $TEST | sed -e 's|.|=|g')" >> $LOG
    echo >> $LOG

    _check_usr_X11 | while read filename ; do
        distinfo -q $filename >> $LOG 2>&1
    done

    RET=$(($RET + 1))
else
    echo "pass"
fi

_check_usr_wayland 2>&1
if [ $? -eq 0 ]; then
    echo "FAIL (see $LOGNAME)"

    echo "Command:" >> $LOG
    echo 'find /usr/wayland -type f -name "*.pc" | grep -v "/usr/wayland/lib/pkgconfig"' >> $LOG
    echo >> $LOG
    echo "$TEST" >> $LOG
    echo "$(echo $TEST | sed -e 's|.|=|g')" >> $LOG
    echo >> $LOG

    _check_usr_wayland | while read filename ; do
        distinfo -q $filename >> $LOG 2>&1
    done

    RET=$(($RET + 1))
else
    echo "pass"
fi

exit $RET
