#!/bin/sh
#
# Verify special ownership and permission information.
#

PATH=/usr/bin:/usr/sbin
CWD="$(pwd)"
TEST="$0"
LOGNAME="$(basename $TEST .sh).log"
LOG="$CWD/$LOGNAME"
RET=0

rm -f $LOG

_check() {
    pathname="$1"
    perms="$2"
    owner="$3"
    group="$4"

    echo "$1" >> $LOG
    echo "$(echo $1 | sed -e 's|.|=|g')" >> $LOG
    echo >> $LOG

    echo -n "checking $pathname exists..." | tee -a $LOG
    stat $pathname >/dev/null 2>&1
    if [ $? -eq 0 ]; then
        echo "ok" | tee -a $LOG
    else
        echo "FAIL" | tee -a $LOG
        echo >> $LOG
        RET=1
        return
    fi
    echo "    stat $pathname >/dev/null 2>&1" >> $LOG
    echo "    [ $? -eq 0 ] && echo ok || echo FAIL" >> $LOG

    echo -n "checking $pathname has $perms permissions..." | tee -a $LOG
    if [ "$perms" = "$(stat -c %a $pathname)" ]; then
        echo "ok" | tee -a $LOG
    else
        echo "FAIL" | tee -a $LOG
        RET=1
    fi
    echo "    [ \"$perms\" = \"\$(stat -c %a $pathname)\" ]" >> $LOG

    echo -n "checking $pathname has user ownership $owner..." | tee -a $LOG
    if [ "$owner" = "$(stat -c %U $pathname)" ]; then
        echo "ok" | tee -a $LOG
    else
        echo "FAIL" | tee -a $LOG
        RET=1
    fi
    echo "    [ \"$owner\" = \"\$(stat -c %U $pathname)\" ]" >> $LOG

    echo -n "checking $pathname has group ownership $group..." | tee -a $LOG
    if [ "$group" = "$(stat -c %G $pathname)" ]; then
        echo "ok" | tee -a $LOG
    else
        echo "FAIL" | tee -a $LOG
        RET=1
    fi
    echo "    [ \"$group\" = \"\$(stat -c %G $pathname)\" ]" >> $LOG

    echo >> $LOG
}

#
# To check a file or directory, add a line beginning with "_check", which is
# the call to the _check() function.  There are four required arguments:
#     ARGUMENTS                  EXAMPLE
#     path name                  /tmp
#     permissions in octal       1777
#     owner name                 root
#     group name                 root
#
# NOTE: For the octal permissions, drop the leading zero if there is one
# because the stat(1) command does not display that on output even though
# we store it in our plist files.
#

_check /var/run 755 root root
_check /tmp 1777 root root
_check /var/tmp 1777 root root

_check /usr/sbin/mount.nfs 4511 root root
_check /var/lib/nfs/sm 700 rpcuser root
_check /var/lib/nfs/sm.bak 700 rpcuser root
_check /var/lib/nfs/state 600 rpcuser root

_check /usr/bin/crontab 4755 root root
_check /var/spool/cron 700 root root

_check /usr/bin/plocate 2711 root locate

_check /usr/sbin/pam_timestamp_check 4755 root root
_check /usr/sbin/unix_chkpwd 4755 root root

_check /usr/X11/bin/Xorg 4711 root root

_check /usr/libexec/pt_chown 4711 root root

_check /usr/bin/fusermount 4755 root root

[ $RET -eq 0 ] && rm -f $LOG

exit $RET
