#!/bin/sh
#
# Look for any sysroot path references in installed files.
#

PATH=/usr/bin:/usr/sbin
CWD="$(pwd)"
TEST="$0"
LOGNAME="$(basename $TEST .sh).log"
LOG="$CWD/$LOGNAME"
TMPDIR="$(mktemp -d)"
RET=0

rm -f $LOG

_check() {
    fail=0
    prevfail=0
    prevd=
    needle="/sysroot"

    echo "checking $1 for any sysroot path references..."

    echo "Command:" >> $LOG
    echo "_check $1" >> $LOG
    echo >> $LOG

    echo "$TEST: $1" >> $LOG
    echo "$(echo "$TEST $1" | sed -e 's|.|=|g')" >> $LOG

    find $1 -not \( -path /usr/src -prune \) \
            -not \( -path /usr/obj -prune \) \
            -not \( -path /usr/distfiles -prune \) \
            -not \( -path /usr/release -prune \) \
            -not \( -path /usr/share/doc -prune \) \
            -not \( -path /usr/share/licenses -prune \) \
            -not \( -path /usr/share/cmake/Help/manual -prune \) \
            -not \( -path /usr/lib/modules -prune \) \
            -type f | \
    grep -v "usr/share/man/man5/pkgconf-personality.5" | \
    grep -v "usr/share/zsh/functions/_gcc" | \
    grep -v "usr/share/man/man1/ostree.1" | \
    grep -v "usr/include/pkg.mk" | \
    grep -v "usr/share/bash-completion/completions/ostree" | \
    grep -v "usr/share/gir-1.0/OSTree-1.0.gir" | \
    grep -v "usr/lib/python2.7/_sysconfigdata.pyc" | \
    grep -v "usr/lib/python2.7/_sysconfigdata.pyo" |
    sort > $TMPDIR/pathlist

    if [ -s $TMPDIR/pathlist ]; then
        while read pathname ; do
            t="$(file -b --mime-type "$pathname")"

            case $t in
                text/*)
                    grep -Hn "$needle" "$pathname" >> $LOG 2>&1
                    if [ $? -eq 0 ]; then
			echo "    * $pathname...references [${needle}]"
                        fail=$(($fail + 1))
		    fi
                    ;;
                application/x-gzip|application/gzip)
                    # These are highly likely to be compressed man pages
                    # Also, why do we compress man pages?  I mean, who
                    # cares these days?
                    gzip -dc "$pathname" | grep -q "$needle" >/dev/null 2>&1
                    if [ $? -eq 0 ]; then
			echo "    * $pathname...references [${needle}]"
                        fail=$(($fail + 1))
                        gzip -dc "$pathname" | grep -n "$needle" | sed -e "s|^|$pathname:|g" >> $LOG 2>&1
                    fi
                    ;;
                *)
                    continue
                    ;;
            esac
        done < $TMPDIR/pathlist
    fi

    return $fail
}

_check /usr
RET=$(($RET + $?))

_check /etc
RET=$(($RET + $?))

rm -rf $TMPDIR
echo
echo -n "Overall $TEST status:"

if [ $RET -gt 0 ]; then
    echo "FAIL"
else
    echo "pass"
    rm -f $LOG
fi

exit $RET
